/*
 * Bootstrap code for the INTEL 
 * $Id: intel_subr.c,v 1.52 2000/10/16 23:59:01 rminnich Exp $
 *
 */

#ifndef lint
static char rcsid[] =
"$Id: intel_subr.c,v 1.52 2000/10/16 23:59:01 rminnich Exp $";
#endif

#include <p5/io.h>

#include <p5/macros.h>
#include <printk.h>
#include <pci.h>
#include <subr.h>

#ifdef SERIAL_CONSOLE
#include <serial_subr.h>
#endif

// kludgy but this is only used here ...
static char *vidmem;		/* The video buffer, should be replaced by symbol in ldscript.ld */
static int line_number;

#define LINES  	25		/* Number of lines and   */
#define COLS   	80		/*   columns on display  */
#define VIDBUFFER 0x20000;

#ifdef VIDEO_SCROLLING
static void scroll()
{
	int i;

	memcpy(vidmem, vidmem + COLS * 2, (LINES - 1) * COLS * 2);
	for (i = (LINES - 1) * COLS * 2; i < LINES * COLS * 2; i += 2)
		vidmem[i] = ' ';
}
#endif

// initialize the display
void displayinit(void)
{
	line_number = 0;
	vidmem = (char *) VIDBUFFER;
	memset(vidmem, 0, 64 * 1024);
#ifdef SERIAL_CONSOLE
#endif
}

void display(char msg[])
{
	int i = 0;

#ifdef SERIAL_CONSOLE
	ttys0_tx_string(msg);
#endif

#ifdef VIDEO_BIOS_WORKS
	while (msg[i] != 0) {
		vidmem[(line_number + 1) * COLS * 2 + i * 2] = msg[i];
		vidmem[(line_number + 1) * COLS * 2 + i * 2 + 1] = 0x07;
		i++;
	}
	(line_number >= (LINES - 1)) ? scroll() : line_number++;
#else /* VIDEO_BIOS_WORKS */
	while (msg[i] != 0)
		vidmem[(line_number++)] = msg[i++];

	/* vidmem[(line_number++)]= 0; */
#endif /* VIDEO_BIOS_WORKS */
}

void printnibl(unsigned long nibl)
{
	static char c[2];
	nibl &= 0xf;
	c[0] = nibl;
	c[1] = 0;
	if ((nibl >= 0) && (nibl <= 9))
		c[0] += '0';
	else
		c[0] += ('A' - 10);
	display(c);

}

void printchar(unsigned char x)
{
	int i;
	for (i = 4; i >= 0; i -= 4)
		printnibl(x >> i);
	display(" ");
}

void printint(unsigned long x)
{
	int i;
	display(" 0x");
	for (i = 28; i >= 0; i -= 4)
		printnibl(x >> i);
	display(" ");
}

void error(char errmsg[])
{
	display(errmsg);

	intel_post(0xee);

	while (1);		/* Halt */
}

/*
 *    Write POST information
 */
void intel_post(char value)
{
	outb(value, 0x80);
}

/*
 *    Write the special configuration registers on the INTEL
 */
void intel_conf_writeb(unsigned long port, unsigned char value)
{
	unsigned char whichbyte = port & 3;
	port &= (~3);
	outl(port, PCI_CONF_REG_INDEX);
	outb(value, PCI_CONF_REG_DATA + whichbyte);
}

/*
 *    Read the special configuration registers on the INTEL
 */
unsigned char intel_conf_readb(unsigned long port)
{
	unsigned char whichbyte = port & 3;
	port &= (~3);
	outl(port, PCI_CONF_REG_INDEX);
	return inb(PCI_CONF_REG_DATA + whichbyte);
}

void intel_cache_on(unsigned long base, unsigned long totalram)
{
	intel_post(0x60);

	/* we need an #ifdef i586 here at some point ... */
	__asm__ __volatile__("mov %cr0, %eax\n\t"
			     "and $0x9fffffff,%eax\n\t"
			     "mov %eax, %cr0\n\t");
	/* turns out cache isn't really on until you set MTRR registers on 
	 * 686 and later. 
	 * NOTHING FANCY. Linux does a much better job anyway. 
	 * so absolute minimum needed to get it going. 
	 */
	/* OK, linux it turns out does nothing. We have to do it ... */
#ifdef i686
	// totalram here is in linux sizing, i.e. units of KB. 
	// set_mtrr is responsible for getting it into the right units!
	intel_set_mtrr(base, totalram);
#endif
	intel_post(0x62);
}

void intel_interrupts_on()
{

	/* this is so interrupts work. This is very limited scope -- 
	 * linux will do better later, we hope ...
	 */
	/* this is the first way we learned to do it. It fails on real SMP 
	 * stuff. So we have to do things differently ... 
	 * see the Intel mp1.4 spec, page A-3
	 */
#ifdef SMP
	unsigned long reg, *regp;
#define SVR 0xfee000f0
#define LVT1 0xfee00350
#define LVT2 0xfee00360
#define APIC_ENABLED 0x100
	regp = (unsigned long *) SVR;
	reg = *regp;
	reg &= (~0xf0);
	reg |= APIC_ENABLED;
	*regp = reg;

	regp = (unsigned long *) LVT1;
	reg = *regp;
	reg &= 0xfffe00ff;
	reg |= 0x5700;
	*regp = reg;

	regp = (unsigned long *) LVT2;
	reg = *regp;
	reg &= 0xfffe00ff;
	reg |= 0x5400;
	*regp = reg;
#else
	unsigned long low, high;
	rdmsr(0x1b, low, high);
	low &= ~0x800;
	wrmsr(0x1b, low, high);
#endif

}

/* much better keyboard init courtesy ollie@sis.com.tw 
   TODO: Typematic Setting, the keyboard is too slow for me */
void pc_keyboard_init()
{
	volatile unsigned char regval;

	/* send cmd = 0xAA, self test 8042 */
	outb(0xaa, 0x64);

	/* empty inut bufferm or any other command/data will be lost */
	while ((inb(0x64) & 0x02))
		intel_post(0);
	/* empty output buffer or any other command/data will be lost */
	while ((inb(0x64) & 0x01) == 0)
		intel_post(1);

	/* read self-test result, 0x55 should be returned form 0x60 */
	if ((regval = inb(0x60) != 0x55))
		return;

	/* enable keyboard interface */
	outb(0x60, 0x64);
	while ((inb(0x64) & 0x02))
		intel_post(2);

	/* send cmd: enable IRQ 1 */
	outb(0x61, 0x60);
	while ((inb(0x64) & 0x02))
		intel_post(3);

	/* reset kerboard and self test  (keyboard side) */
	outb(0xff, 0x60);

	/* empty inut bufferm or any other command/data will be lost */
	while ((inb(0x64) & 0x02))
		intel_post(4);
	/* empty output buffer or any other command/data will be lost */
	while ((inb(0x64) & 0x01) == 0)
		intel_post(5);

	if ((regval = inb(0x60) != 0xfa))
		return;

	while ((inb(0x64) & 0x01) == 0)
		intel_post(6);
	if ((regval = inb(0x60) != 0xaa))
		return;
}

#ifdef SIS630_KEYBOARD
void intel_keyboard_on()
{
	u8 regval;
	struct pci_dev *pcidev;

	/* turn on sis630 keyboard/mouse controller */
	pcidev = pci_find_device(PCI_VENDOR_ID_SI, PCI_DEVICE_ID_SI_503, (void *)NULL);
	if (pcidev != NULL) {
		/* Register 0x47, Keyboard Controller */
		pci_read_config_byte(pcidev, 0x47, &regval);
		/* enable both integrated keyboard and PS/2 mouse controller */
		regval |= 0x0c;
		pci_write_config_byte(pcidev, 0x47, regval);
	}
	pc_keyboard_init();
}
#endif

#ifdef PM133_KEYBOARD
void intel_keyboard_on()
{
	volatile unsigned char regval;

	/*  regval = intel_conf_readb(0x8000385A); */
	/*regval |= 0x01; */
	regval = 0xff;
	intel_conf_writeb(0x8000385A, regval);

	/* disable USB1 */
	intel_conf_writeb(0x80003A3C, 0x00);
	intel_conf_writeb(0x80003A04, 0x00);
	regval = intel_conf_readb(0x80003848);
	regval |= 0x04;
	intel_conf_writeb(0x80003848, regval);

	/* disable USB2 */
	intel_conf_writeb(0x80003B3C, 0x00);
	intel_conf_writeb(0x80003B04, 0x00);
	regval = intel_conf_readb(0x80003885);
	regval |= 0x10;
	intel_conf_writeb(0x80003885, regval);

	pc_keyboard_init();

}
#endif

#ifdef PIIX4E_KEYBOARD
#ifndef NEWPCI
void intel_keyboard_on()
{
	u32 controlbits;
	volatile unsigned char regval, kbready;
	unsigned int devfn = (0x12 << 3) | 3;
#define KBC_EN_DEV11 0x10000000
#define KBC_EIO_EN   0x40000000
	/* oh no, we are setting this below. Fix it later. */
	/* to do -- changed these to PciReadByte */
	regval = intel_conf_readb(0x8000904e);
	regval |= 0x2;
	intel_conf_writeb(0x8000904e, regval);

	/* gosh, the PIIX4E is such a special chip! Not in any good sense! */
	/* we have to set two other bits to make the keyboard work. */
	/* this is related to power management, I guess. */
	/* enable the enables -- I'm not kidding, that's what this is. */
	controlbits = PciReadDword(0, devfn, 0x5c);
	controlbits |= KBC_EN_DEV11;
	PciWriteDword(0, devfn, 0x5c, controlbits);
	/* now enable actual keyboard IO */
	controlbits = PciReadDword(0, devfn, 0x60);
	controlbits |= KBC_EIO_EN;
	PciWriteDword(0, devfn, 0x60, controlbits);
	/* now keyboard should work, ha ha. */
	pc_keyboard_init();
}
#else
void intel_keyboard_on()
{
}
#endif
#endif

#ifdef PIIX4E_NVRAM
void intel_nvram_on()
{
	/*; now set up PIIX4e registers 4e and 4f for nvram access.
	   ; 4e will have value 0xc3, 4f will have value 2
	   ; we are going to PIIX4 function 0; the PIIX4 is device 0x12.
	   ; bit coding is 0x80000000 + 0x9000 (dev 0x12)  + 0x4c (x4 aligned)
	   ; plus 2 or 3 (e or f)
	 */
	/* well, this turns on the 1 MB, but we might as well enable APIC 
	 * access too
	 */
#ifdef NEWPCI
        struct pci_dev *pcidev;

        pcidev = pci_find_device(0x8086, 0x7110, (void *)NULL);
        if (pcidev) pci_write_config_word(pcidev, 0x4e, 0x03c3);
#else
	int devfn;

	devfn = PciFindDevice(0, 0x71108086, 0);
	PciWriteWord(0, devfn, 0x4e, 0x03c3);
#endif
}
#endif /* PIIX4E_NVRAM */


#ifdef SIS630_NVRAM
void intel_nvram_on()
{
	struct pci_dev *pcidev;

	/* turn on sis630 nvram. */
	pcidev = pci_find_device(PCI_VENDOR_ID_SI, PCI_DEVICE_ID_SI_503, (void *)NULL);
	if (pcidev != NULL) {
		/* Enable FFF80000 to FFFFFFFF decode */
		pci_write_config_byte(pcidev, 0x40, 0x33);
		/* Flash can be flashed */
		pci_write_config_byte(pcidev, 0x45, 0x40);
	}

	/* turn off nvram shadow in 0xc0000 ~ 0xfffff */
	pcidev = pci_find_device(PCI_VENDOR_ID_SI, PCI_DEVICE_ID_SI_630, (void *)NULL);
	if (pcidev != NULL) {
		/* read cycle goes to System Memory */
		pci_write_config_word(pcidev, 0x70, 0x1fff);
		/* write cycle goest to System Memory */
		pci_write_config_word(pcidev, 0x72, 0x1fff);
	}
}
#endif /* SIS630_NVRAM */

#ifdef PM133_NVRAM
void intel_nvram_on()
{
	/* the VIA 686A South has a very different nvram setup than the piix4e ... */
	/* turn on ProMedia nvram. */
	/* TO DO: use the PciWriteByte function here. */
	intel_conf_writeb(0x80003843, 0xFF);
}
#endif

/* turn on the floppy. On some chipsets you have to do this. */
#ifdef MUST_ENABLE_FLOPPY
#ifdef SIS630
void enable_floppy()
{
	/* unlock it XXX make this a subr at some point */
	outb(0x87, 0x2e);
	outb(0x01, 0x2e);
	outb(0x55, 0x2e);
	outb(0x55, 0x2e);

	/* now set the LDN to floppy LDN */
	outb(0x7, 0x2e);	/* pick reg. 7 */
	outb(0x0, 0x2f);	/* LDN 0 to reg. 7 */

	/* now select register 0x30, and set bit 1 in that register */
	outb(0x30, 0x2e);
	outb(0x1, 0x2f);

	/* all done. */
	outb(1, 0x2e);
}
#endif /* SIS630 */
#endif /* MUST_ENABLE_FLOPPY */

/* these functions query the hardware to figure out how much ram is in
 * the machine. They then place that information in the parameter block. 
 * This is kind of schizophrenic, but we do it this way because the 
 * functions that actually set the registers are kind of hairy, and 
 * we're not sure they can carry the full burden of also passing on 
 * the size information to the rest of the bootstrap. Besides, querying
 * hardware for ram sizes is trivial. 
 */

#ifdef SIS630
/* table for calculate the DRAM size, the unit is Mega Bytes */
const static ramsizes[16] =
{
	8, 32, 32, 64, 16, 64, 64, 128,
	32, 128, 128, 256, 16, 256, 256, 512
};
#define SIS630_BANKENABLE 0x63
#define SIS630_BANK0 0x60
#define SIS630_BANK1 0x61
#define SIS630_BANK2 0x62
#define SIS630_DIMM_LOCATION_FOR_SMA 0x65
unsigned long sizeram()
{
	struct pci_dev *pcidev;
	unsigned int dimm_slot, dimm_reg, sides;
	unsigned long total_size;
	u8 dram_status, dimm_status;
	u8 sma_enable, sma_size, sma_size_bits, sma_location;

	if ((pcidev = pci_find_device(PCI_VENDOR_ID_SI, PCI_DEVICE_ID_SI_630, NULL)) == NULL)
		return 0;

	pci_read_config_byte(pcidev, SIS630_BANKENABLE, &dram_status);
	dimm_status = dram_status & 0x7;
	sma_enable = dram_status & 0x80;

	pci_read_config_byte(pcidev, SIS630_DIMM_LOCATION_FOR_SMA, &sma_location);
	sma_location &= 0x03;

	/* compute Shared Menory Area (SMA) size in Mega Bytes */
	sma_size_bits = (dram_status >> 4) & 0x7;
	if (sma_size_bits > 5)
		// this is invalid!
		sma_size = 0;
	else {
		sma_size = (2 << sma_size_bits);
	}

	for (dimm_slot = 0, total_size = 0, dimm_reg = SIS630_BANK0;
	     dimm_reg <= SIS630_BANK2; dimm_slot++, dimm_reg++) {
		u8 regval;
		if ((dimm_status & (1 << dimm_slot)) == 0)
			/* this DIMM slot does not have SDRAM installed */
			continue;

		pci_read_config_byte(pcidev, dimm_reg, &regval);

		/* is this DIMM single or double sided  ?? */
		if (regval & 0x20)
			sides = 2;
		else
			sides = 1;

		/* low-order 4 bits are a ram size */
		total_size += (ramsizes[regval & 0xf] * sides);
	}

	/* the total memory visible to the system is physical memory installed minus the
	   amount used by VGA (sma_size) */
	total_size -= sma_size;

	/* return the memory size in KB */
	total_size *= 1024;
	return total_size;
}
#endif /* SIS630 */

#if defined(L440BX) || defined(L440GX)
unsigned long sizeram()
{
	/*
	 * This is written for BX but should work also for GX.
	 */
	unsigned long totalmem;
	unsigned char banks;
#ifdef NEWPCI
        struct pci_dev *pcidev;

	/* pci_find_device is way overkill for the host bridge! 
	 * Plus the BX & GX have different device numbers so it
	 * prevents code sharing.
	 */
	pcidev = pci_find_slot(0, PCI_DEVFN(0,0));
        pci_read_config_byte(pcidev, 0x67, &banks);
#else
	unsigned int devfn;

	devfn = DEVFN(0, 0);
	banks = PciReadByte(0, devfn, 0x67);
#endif /* NEWPCI */
	totalmem = (unsigned long) banks *8 * 1024;
#if defined (L440GX) /* does this apply to the BX? */
	if (banks == 0) {
		totalmem = 0x80000000UL;
	}
#endif
	return totalmem;
}

#endif /* defined(L440BX) || defined(L440GX) */

#ifdef PM133
unsigned long sizeram()
{
	return 0;
}
#endif

#ifdef HAVE_FRAMEBUFFER

#ifdef L440GX
void intel_framebuffer_on()
{
	outl(0x8000a004, 0xcf8);
	outb(0x03, 0xcfc);
}
#endif

#ifdef SIS630
void intel_framebuffer_on()
{
	unsigned long devfn = PCI_DEVFN(0, 0);
	unsigned int bus = 0;
	u8 dramstatus;
	u32 command;

#if 0
	/* No, you can not do this here, but WHY ??? */
	/* TODO:
	   1. clean up the memory footprint to make this work.
	   2. make the share size configurable.
	   3. use the information found from DRAM sizing to select whic DIMM is to be shared */
	pcibios_read_config_byte(0, devfn, SIS630_BANKENABLE, &dramstatus);
	dramstatus |= 0x90;
	pcibios_write_config_byte(0, devfn, SIS630_BANKENABLE, dramstatus);
#endif

	/* enable legacy VGA register (0x3?4, 0x3?5), not actually needed */
	devfn = PCI_DEVFN(2, 0);
	pcibios_read_config_dword(0, devfn, 0x3e, &command);
	command |= 0x08;
	pcibios_write_config_dword(0, devfn, 0x3e, command);
}
#endif	/* SIS630 */

#endif	/* HAVE_FRAMEBUFFER */

// mainboard fixup. 
// so far, only need for the PM133
#ifdef PM133
void intel_mainboard_fixup()
{
	unsigned int devfn;
	unsigned char enables;

	// enable the internal I/O decode on the VIA 686
	// to do: use the pcibios_find function here, instead of 
	// hard coding the devfn. 
	devfn = PCI_DEVFN(7, 0);
	enables = pcibios_read_config_byte(0, devfn, 0x81, &enables);
	enables |= 0x80;
	pcibios_write_config_byte(0, devfn, 0x81, enables);

	// enable com1 and com2. 
	enables = pcibios_read_config_byte(0, devfn, 0x83, &enables);
	// 0x80 is enable com port b, 0x1 is to make it com2, 0x8 is enable com port a as com1
	enables = 0x80 | 0x1 | 0x8 ;
	pcibios_write_config_byte(0, devfn, 0x83, enables);
	// note: this is also a redo of some port of assembly, but we want everything up. 
	// set com1 to 115 kbaud
	// not clear how to do this yet. 
	// forget it; done in assembly. 
	// enable IDE, since Linux won't do it.
	// First do some more things to devfn (7,0)
	// note: this should already be cleared, according to the book. 
	pcibios_read_config_byte(0, devfn, 0x48, &enables);
	printk("IDE enable in reg. 48 is 0x%x\n", enables);
	enables &= ~2; // need manifest constant here!
	printk("set IDE reg. 48 to 0x%x\n", enables);
	pcibios_write_config_byte(0, devfn, 0x48, enables);

	// set default interrupt values (IDE)
	pcibios_read_config_byte(0, devfn, 0x4a, &enables);
	printk("IRQs in reg. 4a are 0x%x\n", enables & 0xf);
	// clear out whatever was there. 
	enables &= ~0xf;
	enables |= 4;
	printk("setting reg. 4a to 0x%x\n", enables);
	pcibios_write_config_byte(0, devfn, 0x4a, enables);
	
	// set up the serial port interrupts. 
	// com2 to 3, com1 to 4
	pcibios_write_config_byte(0, devfn, 0x52, 0x34);

	devfn = PCI_DEVFN(7, 1);
	pcibios_read_config_byte(0, devfn, 0x40, &enables);
	printk("enables in reg 0x40 0x%x\n", enables);
	enables |= 3;
	pcibios_write_config_byte(0, devfn, 0x40, enables);
	pcibios_read_config_byte(0, devfn, 0x40, &enables);
	printk("enables in reg 0x40 read back as 0x%x\n", enables);
	// address decoding. 
	// we want "flexible", i.e. 1f0-1f7 etc. or native PCI
        pcibios_read_config_byte(0, devfn, 0x9, &enables);
        printk("enables in reg 0x9 0x%x\n", enables);
	// by the book, set the low-order nibble to 0xa. 
	enables &= ~0xf;
	// cf/cg silicon needs an 'f' here. 
        enables |= 0xf;
        pcibios_write_config_byte(0, devfn, 0x9, enables);
        pcibios_read_config_byte(0, devfn, 0x9, &enables);
        printk("enables in reg 0x9 read back as 0x%x\n", enables);

	// standard bios sets master bit. 
	pcibios_read_config_byte(0, devfn, 0x4, &enables);
	printk("command in reg 0x4 0x%x\n", enables);
	enables |= 5;
	pcibios_write_config_byte(0, devfn, 0x4, enables);
	pcibios_read_config_byte(0, devfn, 0x4, &enables);
	printk("command in reg 0x4 reads back as 0x%x\n", enables);

	// oh well, the PCI BARs don't work right. 
	// This chip will not work unless IDE runs at standard legacy
	// values. 

	pcibios_write_config_dword(0, devfn, 0x10, 0x1f1);
	pcibios_write_config_dword(0, devfn, 0x14, 0x3f5);
	pcibios_write_config_dword(0, devfn, 0x18, 0x171);
	pcibios_write_config_dword(0, devfn, 0x1c, 0x375);
	pcibios_write_config_dword(0, devfn, 0x20, 0xcc0);


}

#else

#if defined(L440GX) && defined(NEWPCI) && 0

#define rdtsc(low,high) \
     __asm__ __volatile__("rdtsc" : "=a" (low), "=d" (high))

static void __rdtsc_delay(unsigned long loops)
{
	unsigned long bclock, now;
	
	rdtscl(bclock);
	do
	{
		rdtscl(now);
	}
	while((now-bclock) < loops);
}

static void __rdtsc_delay2(unsigned long loops, unsigned pm_io)
{
	unsigned long bclock, now;
	
	rdtscl(bclock);
	do
	{
		outl((1 << 11) | ( 1 << 8), pm_io + 0x00);
		rdtscl(now);
	}
	while((now-bclock) < loops);
}

void intel_mainboard_fixup()
{
	struct pci_dev *pm_pcidev, *host_bridge_pcidev, *nic_pcidev;
	unsigned smbus_io, pm_io;
	unsigned int i, j;
	printk("intel_mainboard_fixup()\n");

#if 1
	pm_pcidev = pci_find_device(0x8086, 0x7113, 0);
	nic_pcidev = pci_find_device(0x8086, 0x1229, 0);
	host_bridge_pcidev = pci_find_slot(0, PCI_DEVFN(0,0));
#endif
#if 1
	pci_write_config_byte(nic_pcidev, 0x3c, 21);
#endif
#if 1
	{
	unsigned char byte;
	unsigned short word;
	unsigned long dword;
	for(i = 0; i < 8; i++) {
		pci_read_config_byte(host_bridge_pcidev, 0x60 +i, &byte);
		printk("DRB[i] = 0x%02x\n", byte);
	}
	pci_read_config_byte(host_bridge_pcidev, 0x57, &byte);
	printk("DRAMC = 0x%02x\n", byte);
	pci_read_config_byte(host_bridge_pcidev, 0x74, &byte);
	printk("RPS = 0x%02x\n", byte);
	pci_read_config_word(host_bridge_pcidev, 0x78, &word);
	printk("PGPOL = 0x%04x\n", word);
	pci_read_config_dword(host_bridge_pcidev, 0x50, &dword);
	printk("NBXCFG = 0x%04x\n", dword);
	}
	
#endif
#if 1

	printk("Reset Control Register\n");
	outb(((inb(0xcf9) & 0x04) | 0x02), 0xcf9);

	printk("port 92\n");
	outb((inb(0x92) & 0xFE), 0x92);

	printk("Disable Nmi\n");
	outb(0, 0x70);

	printk("enabling smbus\n");
#if 0
	smbus_io = NewPciIo(0x10);
#else
	smbus_io = 0xFFF0;
#endif
	pci_write_config_dword(pm_pcidev, 0x90, smbus_io | 1); /* iobase addr */
	pci_write_config_byte(pm_pcidev, 0xd2,  (0x4 << 1) | 1); /* smbus enable */
	pci_write_config_word(pm_pcidev, 0x4, 1); /* iospace enable */


	printk("enable pm functions\n");
#if 0
	pm_io = NewPciIo(0x40);
#else
	pm_io = 0xFF80;
#endif
	pci_write_config_dword(pm_pcidev, 0x40, pm_io | 1); /* iobase addr */
	pci_write_config_byte(pm_pcidev, 0x80, 1);  /* enable pm io address */

	printk("disabling smi\n");
	/* GLBEN */
	outw(0x00,      pm_io + 0x20);
	/* GLBCTL */
	outl((1 << 24), pm_io + 0x28);
	
	printk("Disable more pm stuff\n");
	/* PMEN */
	outw((1 << 8), pm_io + 0x02);
	/* PMCNTRL */
	outw((0x5 << 10) , pm_io + 0x4);
	/* PMTMR */
	outl(0, pm_io + 0x08);
	/* GPEN */
	outw(0, pm_io + 0x0e);
	/* PCNTRL */
	outl(0, pm_io + 0x10);
	/* GLBSTS */
	/* DEVSTS */
	/* GLBEN see above */
	/* GLBCTL see above */
	/* DEVCTL */
	outl(0, pm_io + 0x2c);
	/* GPIREG */
	/* GPOREG */
	
	printk("Set the subsystem vendor id\n");
	pci_write_config_word(host_bridge_pcidev, 0x2c, 0x8086);

	printk("Disabling pm stuff in pci config space\n");

#define MAX_COUNTERS
#ifndef MAX_COUNTERS
	/* counters to 0 */
#define WHICH_COUNTERS(min,max) min
#else
	/* max out the counters */
#define WHICH_COUNTERS(min,max) max
#endif

	/* CNTA */
	pci_write_config_dword(pm_pcidev, 0x44, WHICH_COUNTERS(0x004000f0, 0xFFFFFFFF));

	/* CNTB */
	pci_write_config_dword(pm_pcidev, 0x48, WHICH_COUNTERS(0x00000400, 0x007c07df));

	/* GPICTL */
	pci_write_config_dword(pm_pcidev, 0x4c, 0);

	/* DEVRESD */
	pci_write_config_dword(pm_pcidev, 0x50, 0);
	
	/* DEVACTA */
	pci_write_config_dword(pm_pcidev, 0x54, 0);

	/* DEVACTB */
	pci_write_config_dword(pm_pcidev, 0x58, 0);

	/* DEVRESA */
	pci_write_config_dword(pm_pcidev, 0x5c, 0);

	/* DEVRESB */
	pci_write_config_dword(pm_pcidev, 0x60, 0);

	/* DEVRESC */
	pci_write_config_dword(pm_pcidev, 0x64, 0); /* might kill the serial port */

	/* DEVRESE */
	pci_write_config_dword(pm_pcidev, 0x68, 0);

	/* DEVRESF */
	pci_write_config_dword(pm_pcidev, 0x6c, 0);

	/* DEVRESG */
	pci_write_config_dword(pm_pcidev, 0x70, 0);

	/* DEVRESH */
	pci_write_config_dword(pm_pcidev, 0x74, 0);

	/* DEVRESI */
	pci_write_config_dword(pm_pcidev, 0x78, 0);

	/* DEVRESJ */
	pci_write_config_dword(pm_pcidev, 0x7c, 0);

#endif

#if 1
	
	/* Verify that smi is disabled */
	printk("Testing SMI\r\n");
	{
	u32 value;
	pci_read_config_dword(pm_pcidev, 0x58, &value);
	pci_write_config_dword(pm_pcidev, 0x58, value | (1 << 25));
	}
	outb(inb(0xb2), 0xb2);
	printk("SMI disabled\r\n");
#endif
#if 0
	
	for(i = 0; i < 255; i++) {
		printk("%08x\r\n", i);
		__rdtsc_delay2(1000000000UL, pm_io);
	}
#endif
}

#else /* L440GX && NEWPCI */

void intel_mainboard_fixup()
{
}

#endif /* L440GX && NEWPCI */
#endif /* PM133 */

#ifdef NEWPCI
/* These functions should be chip-set independent -tds */
void intel_zero_irq_settings(void)
{
	struct pci_dev *pcidev;
//	unsigned char irq;
	unsigned char line;
  
	pcidev = pci_devices;
  
	while (pcidev) {
		pci_read_config_byte(pcidev, 0x3d, &line);
		if (line) {
			pci_write_config_byte(pcidev, 0x3c, 0);
		}
		pcidev = pcidev->next;
	}
}

void intel_check_irq_routing_table(void)
{
#if (defined(L440BX) || defined (SIS630))
	const u8 *addr;
	const struct irq_routing_table *rt;
	int i;
	u8 sum;

	rt = &intel_irq_routing_table;
	addr = (u8 *)rt;

	sum = 0;
	for (i=0; i<rt->size; i++)
		sum += addr[i];

	printk(KERN_DEBUG "%s:%6d:%s() - "
	       "irq_routing_table located at: 0x%08x\n",
	       __FILE__, __LINE__, __FUNCTION__, addr);

	sum = (unsigned char)(rt->checksum-sum);

	if (sum != rt->checksum) {
		printk(KERN_WARNING "%s:%6d:%s() - "
		       "checksum is: 0x%02x but should be: 0x%02x\n",
		       __FILE__, __LINE__, __FUNCTION__, rt->checksum, sum);
	}

	if (rt->signature != PIRQ_SIGNATURE || rt->version != PIRQ_VERSION ||
	    rt->size % 16 || rt->size < sizeof(struct irq_routing_table)) {
		printk(KERN_WARNING "%s:%6d:%s() - "
		       "Interrupt Routing Table not valid\n",
		       __FILE__, __LINE__, __FUNCTION__);
		return;
	}

	sum = 0;
	for (i=0; i<rt->size; i++)
		sum += addr[i];

	if (sum) {
		printk(KERN_WARNING "%s:%6d:%s() - "
		       "checksum error in irq routing table\n",
		       __FILE__, __LINE__, __FUNCTION__);
	}
#endif /* #if defined(L440BX) */
}

#define RTABLE_DEST 0xf0000

void intel_copy_irq_routing_table(void)
{
#ifdef SIS630
	memcpy((char *) RTABLE_DEST, &intel_irq_routing_table, intel_irq_routing_table.size);
#endif
}

#endif /* NEWPCI */
