/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: stpc_crt0.S,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#include <rom/asm.h>
#include <rom/stpc.h>

#include "stpc_conf.h"

#define	STPC_PDATA_MAGIC0	0xdead
 
/*
 * This is the entry code (the mkrom(8) utility makes a jumpvector
 * to this adddess. 
 *
 * When we get here we are in x86 real mode.
 *
 *	%cs	= 0xf000	%ip	= 0x0000
 *	%ds	= 0x0000	%es	= 0x0000
 *	%dx	= 0x0yxx	(y = 3 for i386, 5 for pentium, 6 for P6,
 *					where x is undefined)
 *	%fl	= 0x0002
 */
 	.text
	.code16
 	.org	0x10000		/* FIXME should not be hardcoded */
	.globl	EXT(_start)
EXT(_start):
	cli

	movw	%cs, %ax
	movw	%ax, %ds
	
	stpc_chip_post_macro(0x01)	/* post 01 */
	
	/** initialize the chip */
	movl	$0f, %sp
	jmp	EXT(stpc_chip_init)	
0:

	stpc_chip_post_macro(0x02)	/* post 02 */

	/** set DRAM refresh */
	movl	$0f, %sp
	jmp	EXT(stpc_chip_refresh)
0:

/* 
 *	Set memory bank sizes.
 */
	stpc_chip_post_macro(0x03)	/* post 03 */

#ifdef	STPC_DRAM_AUTOSIZE
	/*
	 * If we does autosizing of the memory banks we set
	 * them to 2MB (1?) for now.
	 */
	movb	$0x01, %al		/* 2 MB */
	movb	$0x30, %ah
	movw	$3, %cx
.Lsetmb:
	stpc_conf_write_macro
	inc	%ah
	loop	.Lsetmb
	
	/** set width to 32-bit */
	movb	$0x34, %ah
	stpc_conf_read_macro
	andb	$ ~0b01111, %al
	orb	$0b1111, %al		/* all banks 32-bit */
	stpc_conf_write_macro

#else
	/*
	 * Set memory bank top addresses for all the memory banks:
	 */
	movb	$STPC_MB0_TOPADDR, %al	/* memory bank 0 */
	movb	$0x30, %ah
	stpc_conf_write_macro

	movb	$STPC_MB1_TOPADDR, %al	/* memory bank 1 */
	movb	$0x31, %ah
	stpc_conf_write_macro
	
	movb	$STPC_MB2_TOPADDR, %al	/* memory bank 2 */
	movb	$0x32, %ah
	stpc_conf_write_macro

	movb	$STPC_MB3_TOPADDR, %al	/* memory bank 3 */
	movb	$0x33, %ah
	stpc_conf_write_macro

	/*
	 * Set memory bank width for all the memory banks:
	 */
	movb	$0x34, %ah
	stpc_conf_read_macro
	andb	$ ~0b01111, %al
	orb	$STPC_MB_WIDTH, %al
	stpc_conf_write_macro
#endif

/*
 *	Setup DRAM type and speed same for all banks.
 */
	stpc_chip_post_macro(0x05)	/* post 05 */

 	xorw	%bx, %bx
	xorb	%dh, %dh
.Lnext:	movb	$STPC_DRAM_EDO_70, %dl	/* set slowest for now */
	movl	$0f, %esp
	jmp	EXT(stpc_chip_setmtype)	/* set memory timings */
0:	incw	%bx
	cmpw	$3, %bx	
	jbe	.Lnext			/* 4 banks */


/* 
 *	Graphics memory size register (0x36).
 *
 *	This register defines the size of DRAM used by graphics
 *	for frame buffer. 
 *
 *	Bits 0-5 (Top of Graphics Memory) indicates frame buffer 
 *	size in 128KB units. The range is 0 to 32 for 0 to 4MB 
 *	framebuffer, so 6 bits are necessary.
 */
#if (STPC_FRAME_BUF_SZ / 128) > 33
#error	"frame buffer size >= 4MB)"
#endif

	stpc_chip_post_macro(0x04)	/* post 04 */

	movb	$0x36, %ah
	movb	$STPC_FRAME_BUF_SZ>>8, %al
	stpc_conf_write_macro

/*
 *	Set ISA clock and other misc config 0
 */
	/* set isa clock (PCI clk / 4) */
	movb	$0x50, %ah
	stpc_conf_read_macro
	orb	$0x10, %al
	stpc_conf_write_macro
	
	/* enable the ISA synchroniser (ISA PCI sync) */
	movb	$0x59, %ah
	stpc_conf_read_macro
	andb	$~0x1, %al
	stpc_conf_write_macro

	/* ISA write post enable, isa read buffer enable */
	movb	$0x50, %ah
	stpc_conf_read_macro
	andb	$ ~0b11110000, %al
	orb	$0x80 | 0x40, %al	/* post write, buff read */ 
	stpc_conf_write_macro

/*
 *	Setup optimized chipset settings befor shadow
 */
	stpc_chip_post_macro(0x06)	/* post 06 */

	movb	$0x28, %ah		/* Shadow Control register 3 - C.I. 28 */
	movb	$0x20, %al		/* cacheability enabled for seg F000 */
	stpc_conf_write_macro

	movb	$0x81, %ah
	movb	$0x22, %al
	stpc_conf_write_macro

/*
 *	Setup a stack now, since the memory is initialized.
 *
 *	We use the first page in memory as stack for now. All above can be
 *	used as user memory. 
 *
 *	We expand the stack as fast as we have switched to protected mode.
 */
	stpc_chip_post_macro(0x07)	/* post 07 */

 	xorw	%ax, %ax
 	movw	%ax, %ss
	movl	$0x1000, %esp

/*
 *	Copy private startup code data to the specified _PDATABASE segment. 
 */
	stpc_chip_post_macro(0x08)	/* post 08 */

	movw	%cs, %ax		/* setup source segment */
	movw	%ax, %ds

	xorw	%ax, %ax		/* setup destination segment */
	movw	%ax, %es

	movl	$EXT(_PDATABASE), %edi	/* destination offset */
	movl	$EXT(_pdata), %esi	/* source offset */
	movw	$0x1000 >> 2, %cx	/* number of dwords */

	cld
	rep
	movsl	

/*
 *	Here we do some memory tests. First we see if a write to the
 *	memory works. If not; panic. We also tests if the private data
 *	has been copied to the memory.
 *
 *	Error:	pdata test:	3 beeps
 */

	xorw	%ax, %ax
	movw	%ax, %ds

0:	cmpw	$STPC_PDATA_MAGIC0, EXT(pdata_magic)
	je	.Lpdataok
	movw	$3, %dx
	jmp	EXT(stpc_panic)

.Lpdataok:

	/** switch to protected mode */
	stpc_chip_post_macro(0x09)	/* post 09 */

	movl	%cr0, %eax
	orl	$1, %eax
	movl	%eax, %cr0

	/** load gdt */
	stpc_chip_post_macro(0x0a)	/* post 0a */
	lgdt	EXT(gdtptr)

	stpc_chip_post_macro(0x0b)	/* post 0b */
	data32
	ljmp	$8, $.Lprotected

/*
 *	When we come here we are in protected mode. We expand 
 *	the stack and copies the data segment from ROM to the
 *	memory.
 *
 *	After that, we call the chipset bootstrap routine that
 *	does what is left of the chipset initialization. 
 *
 *	NOTE aligned to 4 so that we are sure that the prefetch
 *	cache will be reloaded.
 */
	.align	4
	.code32
.Lprotected:
	stpc_chip_post_macro(0x10)	/* post 10 */

	movw	$0x10, %ax
	movw	%ax, %ds
	movw	%ax, %es
	movw	%ax, %ss
	
	/* set new stack */
	movl	$_PDATABASE, %esp
	
/*
 *	Copy data into RAM and clear the BSS. Since these segments
 *	isn't really that big we just copy/clear using bytes, not
 *	double words.
 */
	stpc_chip_post_macro(0x11)	/* post 11 */

	cld				/* clear direction flag */
	leal	EXT(_ldata), %esi
	leal	EXT(_data), %edi
	movl	$EXT(_eldata), %ecx
	subl	%esi, %ecx
	jz	.Lnodata		/* should not happen */
	rep
	movsb
.Lnodata:
	stpc_chip_post_macro(0x12)	/* post 12 */
	
	/** clear bss */
	leal	EXT(_bss), %edi
	movl	$EXT(_ebss), %ecx
	subl	%edi, %ecx
	jz	.Lnobss
	xorl	%eax, %eax
	rep
	stosb
.Lnobss:
	
/*
 *	Initiailize the last things.
 */
	stpc_chip_post_macro(0x13)	/* post 13 */

	call	EXT(stpc_bootstrap)

/*
 *	Turn the on the cache!
 *
 */
#if 0
 	movb	$0x20, %ah
	stpc_conf_read_macro
	orb	$3, %al
	stpc_conf_write_macro
#endif

/*
 *	Now we are finished. Memory is up, data is copied and
 *	bss is cleared.   Now we call the ``main routine and
 *	let it do the rest.
 */ 
	stpc_chip_post_macro(0xfe)	/* post fe */

	call	EXT(main)
	/*NOTREACHED*/
.Lhlt:	hlt
	jmp	.Lhlt


/*
 *	stpc_panic:
 *
 *	A simple routine that beeps and halts the CPU. The number of times
 *	that the CPU should beep is specified in the %dx register.
 */
STATIC(stpc_panic)
	movb	$0xb6, %al
	outb	%al, $0x43
	.word	0xe3
	movb	$5, %al
	outb	%al, $0x42
	.word	0xe3
	outb	%al, $0x42

.Lsoundon:
	movb	$0xf, %al
	outb	%al, $0x61

	movl	$0xC0000, %cx
0:	loop	0b

	movb	$0xc, %al
	outb	%al, $0x61

	movl	$0xC0000, %cx
0:	loop	0b

	dec	%dx
	jnz	.Lsoundon

0:	jmp	0b

/*
 *	A
 */
GLOBAL(stpc_bell)
	movb	$0xb6, %al
	outb	%al, $0x43
	.word	0xe3
	movb	$5, %al
	outb	%al, $0x42
	.word	0xe3
	outb	%al, $0x42

.Lsoundon0:
	movb	$0xf, %al
	outb	%al, $0x61

	movl	$0xC0000, %cx
0:	loop	0b

	movb	$0xc, %al
	outb	%al, $0x61
	ret

/*
 *	This is private data to the startup code. This data
 *	is relocated at _PDATABASE and are copied there as fast
 *	as the memory is availible. This data is used to 
 *	get the system into 32-bit protected mode. IDT and other
 *	stuff is up to the 32-bit code to initialize.
 */
	.section .pdata, "aw"		/* ALLOC, DATA */
	.globl	EXT(gdt)
STATIC(gdt)
	.word	0x0000, 0x0000		/* dummy */
	.byte	0x0, 0x0, 0x0, 0x0
	
	.word	0xffff, 0x0000		/* flat code segment */
	.byte	0x0, 0x9a, 0xcf, 0x0	
	
	.word	0xffff, 0x0000		/* flat data segment */
	.byte	0x0, 0x92, 0xcf, 0x0	

	.word	0x0000, 0x0000		/* kernel tss */
	.byte	0x0, 0x0, 0x0, 0x0

STATIC(gdtptr)	
	.word	3*8-1
	.long	EXT(gdt)		/* we know the offset */

STATIC(pdata_magic)
	.word	STPC_PDATA_MAGIC0
