/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: stpc_subr.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: stpc_subr.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <sys/types.h>
#include <rom/stpc.h>
#include "stpc_conf.h"

/** from subr_locore.S: */
extern void outb(int, char);
extern char inb(int);

/*
 *	Write POST information
 */
void stpc_post(
	char	value)
{
	outb(0x80, value);
}

/*
 *	Write the special configuration registers on the STPC
 */
void stpc_conf_write(
	char	port,
	char	value)
{
	outb(STPC_CONF_REG_INDEX, port);
	outb(STPC_CONF_REG_DATA, value);
}

/*
 *	Read the special configuration registers on the STPC
 */
char stpc_conf_read(
	char	port)
{
	outb(STPC_CONF_REG_INDEX, port);
	return inb(STPC_CONF_REG_DATA);
}

/*
 *	A *small* dealy.
 */
#define stpc_delay()	__asm__("jmp 0f ; 0:")

#define	STPC_DRAM_SZMB_MAX	0x7f		/* 128MB */
#define	STPC_DRAM_MAGIC		0xa55aa55a	/* magic value */
/*
 *
 */
void
stpc_autosize()
{

	/*
	 * If we use some form of autosizing, this is the time to 
	 * find out how much memory we have.
	 */
	{
		unsigned long *s = 0x0, *d = 0x0;
		int cnt, top = 0, b64 = 0, tag = 0, sz;

		/** set VGA frame to 0K default */
		stpc_conf_write(0x36, 0);
		
		/** set all banks top address to 128 MB */
		for(cnt = 0; cnt < 3; cnt++) 
			stpc_conf_write(0x30 + cnt, STPC_DRAM_SZMB_MAX);

#ifdef STPC_DRAM_64BIT
		/** set all banks width to 64-bit */
		stpc_conf_write(0x34, stpc_conf_read(0x34) & 0xf0);	/* clear bit 3..0 */
#endif

		stpc_post(0x21);	/* post 21 */

		/*
		 * Loop through all the memory banks and try to find out
		 * how large they are and their width (if 64-bit is enabled
		 * that is).
		 */
		for(cnt = 0; cnt < 4; cnt++) {
			stpc_post(0x22+cnt);	/* post 22+x */

			*(s)	=  STPC_DRAM_MAGIC;
			stpc_delay();
#ifdef STPC_DRAM_64BIT
			/** find if bank is 32/64 bit */
			*(s+4)	= ~STPC_DRAM_MAGIC;
			stpc_delay();
#endif
	
			/*
			 * Check if the bank is empty or not. If it isn't empty,
			 * lets find the size of the bank by running through it
			 * and see when it wraps back.
			 */		
			if(*s == STPC_DRAM_MAGIC) {
#ifdef STPC_DRAM_64BIT
				if(*(s + 4) != ~STPC_DRAM_MAGIC) 
					stpc_conf_write(0x34, stpc_conf_read(0x34) | (1 << cnt));	/* set 32-bit */
#endif

				stpc_post(0x30);	/* post 30 */

				for(d = s, tag = 0, sz = 0; top != STPC_DRAM_SZMB_MAX; ++tag, ++top, ++sz) {
					d  = d + 0x100000;	/* advance 1 MB */
					*d = (unsigned long) tag;
					stpc_delay();
					if(*s = (unsigned long) tag);
						break;
				}
			}

			/*
			 * Some sanity checking. If bank 0 is empty, we got a el bigo problemo.
			 * Really, this should never happen, since we're has already copied data
			 * to that memory and if there's no memory there, the computer should
			 * already be dead or doing some really nasty stuff. 
			 */
			else if(cnt == 0) {
				stpc_post(0x31);	/* post 14 */
				/*PANIC: no memory in bank0 */
				for(;;);
			}
				
			/*
			 * Set the size of the memory bank. 
			 */
			stpc_post(0x32);	/* post 15 */

			printf("bank %d: %d\n", cnt, top);
		
			if(cnt == 0) 
				--top;
			stpc_conf_write(0x30 + cnt, top);
			s += (sz << 20);
		}	
	}

}

/*
 *	STPC initialize routine.
 */
void 
stpc_bootstrap(
	void)
{

#ifdef STPC_DRAM_AUTOSIZE
#define	STPC_DRAM_SZMB_MAX	0x7f		/* 128MB */
#define	STPC_DRAM_MAGIC		0xa55aa55a	/* magic value */

	/*
	 * If we use some form of autosizing, this is the time to 
	 * find out how much memory we have.
	 */
	{
		unsigned long *s = 0x0, *d = 0x0;
		int cnt, top = 0, b64 = 0, tag = 0, sz;

		stpc_post(0x20);	/* post 20 */

		/** set VGA frame to 0K default */
		stpc_conf_write(0x36, 0);
		
		/** set all banks top address to 128 MB */
		for(cnt = 0; cnt < 3; cnt++) 
			stpc_conf_write(0x30 + cnt, STPC_DRAM_SZMB_MAX);

#ifdef STPC_DRAM_64BIT
		/** set all banks width to 64-bit */
		stpc_conf_write(0x34, stpc_conf_read(0x34) & 0xf0);	/* clear bit 3..0 */
#endif

		stpc_post(0x21);	/* post 21 */

		/*
		 * Loop through all the memory banks and try to find out
		 * how large they are and their width (if 64-bit is enabled
		 * that is).
		 */
		for(cnt = 0; cnt < 4; cnt++) {
			stpc_post(0x22+cnt);	/* post 22+x */

			*(s)	=  STPC_DRAM_MAGIC;
			stpc_delay();
#ifdef STPC_DRAM_64BIT
			/** find if bank is 32/64 bit */
			*(s+4)	= ~STPC_DRAM_MAGIC;
			stpc_delay();
#endif
	
			/*
			 * Check if the bank is empty or not. If it isn't empty,
			 * lets find the size of the bank by running through it
			 * and see when it wraps back.
			 */		
			if(*s == STPC_DRAM_MAGIC) {
#ifdef STPC_DRAM_64BIT
				if(*(s + 4) != ~STPC_DRAM_MAGIC) 
					stpc_conf_write(0x34, stpc_conf_read(0x34) | (1 << cnt));	/* set 32-bit */
#endif

				stpc_post(0x30);	/* post 30 */

				for(d = s, tag = 0, sz = 0; top != STPC_DRAM_SZMB_MAX; ++tag, ++top, ++sz) {
					d  = d + 0x100000;	/* advance 1 MB */
					*d = (unsigned long) tag;
					stpc_delay();
					if(*s = (unsigned long) tag);
						break;
				}
			}

			/*
			 * Some sanity checking. If bank 0 is empty, we got a el bigo problemo.
			 * Really, this should never happen, since we're has already copied data
			 * to that memory and if there's no memory there, the computer should
			 * already be dead or doing some really nasty stuff. 
			 */
			else if(cnt == 0) {
				stpc_post(0x31);	/* post 14 */
				/*PANIC: no memory in bank0 */
				for(;;);
			}
				
			/*
			 * Set the size of the memory bank. 
			 */
			stpc_post(0x32);	/* post 15 */

			if(cnt == 0) 
				--top;
			stpc_conf_write(0x30 + cnt, top);
			s += (sz << 20);
		}	
	}
#endif
}

/*
 * HAL routines:
 */
int hal_cpu_speed(
		void)
{
	static int table[] = {		/* for STPC client */
		0, 0, 0, 25, 50, 60, 66, 75 };
	unsigned char value;
	
	value = stpc_conf_read(0x5f);
	
	return table[(value >> 3) & 0x7]; 
}

char *hal_cpu_type(
		void)
{
	static char buff[32];
	unsigned char value, *clk;
	
	value = stpc_conf_read(0xfe);
	switch(value) {
		case 0x1a:
		case 0x1f:
			clk = "1X Clock";
			break;
		
		case 0x1b:
			clk = "2X Clock";
			break;
	}
	
	sprintf(buff, "STPC, %s, %dMHz CPU bus frequency",
		clk, hal_cpu_speed());
	return buff;
}

void hal_memory_type(
		void)
{
	char *types[] = { "EDO", "FPM" };
	char *speeds[] = { "70", "60" };
	unsigned char value;
	int cnt;
	
	for(cnt = 0; cnt < 2; cnt++) {
		value = stpc_conf_read(0x4a+cnt);
		
		printf("*** bank %d %s (%s ns)\n",
			(cnt*2), 
			types[(value >> 5) & 1],
			speeds[(value >> 7) & 1]);
		printf("*** bank %d %s (%s ns)\n",
			(cnt*2)+1, 
			types[(value >> 2) & 1],
			speeds[(value >> 4) & 1]);
	}
}
