/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: pciide.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: pciide.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <rom/bus/bus.h>
#include <rom/bus/pci.h>
#include <rom/device.h>

#include <device/pciide.h>

/*
 *	This is a generic PCI device driver. It is used to configure
 *	unknown devices on the PCI bus. 
 */

/*
 *	PCI device private data:
 */
typedef struct pciide_softc {
	bus_t			sc_bus;		/* our bus */
	pci_tag_t		sc_tag;		/* pci tag */
} *pciide_softc_t;

/*
 *	Routine 0 when found, else !0
 */
int pciideprobe(
	device_t	parent,
	cfdata_t	cf,
	void		*aux)
{
	pci_attach_args_t pa = (pci_attach_args_t) aux;

	if(pa->pa_bargs.ba_bus->bus_type != BUS_TYPE_PCI)
		return CF_MATCH_NONE;

	if(pa->pa_vendorid == 0x100e && pa->pa_productid == 0x55cc
	   && pa->pa_class == PCI_CLASS_MASS_STORAGE
	   && pa->pa_subclass == PCI_SUBCLASS_MASS_STORAGE_IDE)
		return CF_MATCH_EXACT;
	return CF_MATCH_NONE;
}

/*
 *
 */
int32_t pciideattach(
	device_t	parent,
	device_t	self,
	void		*aux)
{
	pciide_softc_t	sc = (pciide_softc_t) self->dv_softc;
	pci_attach_args_t pa = (pci_attach_args_t) aux;
	int		i, rval;
	unsigned char	value;
	bus_t		bus;
	
	/*
	 * We have two channels: primary and secondary.
	 */
	int channel, nchannels = NCHANNELS;
	
	/*
	 *
	 */
	int drive, ndrives = NDRIVES;
	
	/*
	 *
	 */
	int interface;
	
	/*
	 * Print some information about the device.
	 */	
	device_print(self, pa);	
	printf(": pci-to-ide bridge");

	/*
	 * Copy arguments.
	 */
	sc->sc_tag	= pa->pa_tag;

	/*
	 *
	 */
	value = pci_conf_read_1(sc->sc_tag, 
		PCI_CONF_REGISTER_COMMAND);
	if((value & PCI_COMMAND_IO_ENABLE) == 0) {
		pci_conf_write_1(sc->sc_tag,
				PCI_CONF_REGISTER_COMMAND,
				PCI_COMMAND_IO_ENABLE);		

		value = pci_conf_read_1(sc->sc_tag,
			PCI_CONF_REGISTER_COMMAND);

		if((value & PCI_COMMAND_IO_ENABLE) == 0) {
			printf(", failed seting I/O enable");
		}
		else
		printf(", enabling I/O");
	}


#if 0
	/*
	 * Set the IDE bridge into native mode.
	 */
	pci_conf_write_1(sc->sc_tag,
			0x09,		/* MACRO HERE PLEASE */
			SB_INTERFACE_SECONDARY_MODE_NATIVE
			| SB_INTERFACE_PRIMARY_MODE_NATIVE);
	
	printf(", puting bridge into native mode");
	
	value = pci_conf_read_1(sc->sc_tag,
			0x09);
	if((value & 5) != 5)
		printf(" (FAILED!)");

	/** initialize primary */
	pci_conf_write_4(sc->sc_tag,
			0x10,
			PCIIDE_IOBASE_PRIMARY);
	pci_conf_write_4(sc->sc_tag,
			0x14,
			PCIIDE_IOBASE_PRIMARY_AUX);

	/** initiailize secondary */
	pci_conf_write_4(sc->sc_tag,
			0x18,
			PCIIDE_IOBASE_SECONDARY);
	pci_conf_write_4(sc->sc_tag,
			0x1c,
			PCIIDE_IOBASE_SECONDARY_AUX);
#else
	pci_conf_write_1(sc->sc_tag,
			0x09,
			SB_INTERFACE_SECONDARY_MODE_LEGACY
			| SB_INTERFACE_PRIMARY_MODE_LEGACY); 
	printf(", puting bridge into legacy mode");
#endif
	printf("\n");

	interface = pci_conf_read_1(sc->sc_tag,
			0x09);
	for(channel = 0; channel < nchannels; channel++) {
		struct pciide_attach_args pi;

		printf("%s: %s channel %s to %s mode\n",
			self->dv_name, PCIIDE_CHAN_NAME(channel),
			(interface & PCIIDE_INTERFACE_SETTABLE(channel)) ?
				"configured" : "wired",
			(interface & PCIIDE_INTERFACE_PCI(channel)) ? 
				"native-PCI" : "compatibility");
		
			
		/* 
		 * We probe per channel. 
		 */
		pi.pi_cmdtag	= isa_maketag(PCIIDE_CHAN_IOBASE_CMD(channel));
		pi.pi_auxtag	= isa_maketag(PCIIDE_CHAN_IOBASE_AUX(channel));
		pi.pi_channel	= channel;

		rval = wdc_probe(&pi, 6000);
		if(rval == 0) {
			printf("%s: %s channel ignored\n",
				self->dv_name, PCIIDE_CHAN_NAME(channel));
			continue;
		}
	
		for(drive = 0; drive < ndrives; drive++) {
			pi.pi_drive	= drive;
	
			if((rval & (1 << drive)) == 0)
				continue;
			
			(void) config_search(self,
					0,
					(void *) &pi);
		}
	}
	return 0;
}

/*
 *	Device configration data:
 */
struct cfattach ca_pci_pciide = {
	sizeof(struct pciide_softc),
	pciideprobe,	/* match */
	pciideattach,	/* attach */
	0,		/* detach */
	"pciide"	/* name */
};
