/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: pciisa.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: pciisa.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <rom/bus/bus.h>
#include <rom/bus/pci.h>
#include <rom/device.h>

/*
 *	This is a generic PCI device driver. It is used to configure
 *	unknown devices on the PCI bus. 
 */

/*
 *	PCI device private data:
 */
typedef struct pciisa_softc {
	bus_t			sc_bus;		/* our bus */
	pci_tag_t		sc_tag;		/* pci tag */
} *pciisa_softc_t;

/*
 *	Routine 0 when found, else !0
 */
int pciisaprobe(
	device_t	parent,
	cfdata_t	cf,
	void		*aux)
{
	pci_attach_args_t pa = (pci_attach_args_t) aux;

	if(pa->pa_bargs.ba_bus->bus_type != BUS_TYPE_PCI)
		return CF_MATCH_NONE;

	if(pa->pa_vendorid == 0x100e && pa->pa_productid == 0x55cc
	   && pa->pa_class == PCI_CLASS_BRIDGE 
	   && pa->pa_subclass == PCI_SUBCLASS_BRIDGE_ISA)
		return CF_MATCH_EXACT;
	return CF_MATCH_NONE;
}

/*
 *
 */
int32_t pciisaattach(
	device_t	parent,
	device_t	self,
	void		*aux)
{
	pciisa_softc_t	sc = (pciisa_softc_t) self->dv_softc;
	pci_attach_args_t pa = (pci_attach_args_t) aux;
	unsigned	div, iobase;
	bus_t		bus;
	
	/*
	 *
	 */
	u_int8_t misc0;
	
	/*
	 * Print some information about the device.
	 */	
	device_print(self, pa);	
	printf(": pci-to-isa bridge\n");

	/*
	 * Copy information.
	 */
	sc->sc_tag = pa->pa_tag;
	
	/*
	 * Set the PCI-to-ISA bridge in PCI 2.1 mode.
	 */
#define	PCIISA_REGISTER_MISC0	0x40
#define	PCIISA_STANDARD_PCI20	0x01
#define	PCIISA_STANDARD_PCI21	0x00

	(void) pci_conf_write_1(sc->sc_tag,
			PCIISA_REGISTER_MISC0,
			PCIISA_STANDARD_PCI21);
	misc0 = pci_conf_read_1(sc->sc_tag,
			PCIISA_REGISTER_MISC0);

	printf("%s: configured to support the PCI %s standard\n",
		self->dv_name, (misc0 & PCIISA_STANDARD_PCI20) ?
			"2.0" : "2.1");
	return 0;
}

/*
 *	Device configration data:
 */
struct cfattach ca_pci_pciisa = {
	sizeof(struct pciisa_softc),
	pciisaprobe,	/* match */
	pciisaattach,	/* attach */
	0,		/* detach */
	"pciisa"	/* name */
};
