/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: wdc.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: wdc.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <sys/cdefs.h>
#include <sys/types.h>

#include <rom/device.h>

#include <device/wdc.h>
#include <device/pciide.h>

#define	WDCDEBUG_ALL	0x0000
#define	WDCDEBUG_WAIT	0x0001
#define	WDCDEBUG_READ	0x0002
#define	WDCDEBUG_XFER	0x0004
#define	WDCDEBUG_EXEC	0x0008
#define	WDCDEBUG_MALL	(WDCDEBUG_WAIT | WDCDEBUG_READ | WDCDEBUG_XFER | WDCDEBUG_EXEC)

#define	ENABLE_MULTI	1

#if 0
#define	WDC_DEBUG
#endif

u_int16_t	wdc_debug_mask = 0;

#if 0
#define	WDC_PRINTF(b, a)	if(b == WDCDEBUG_EXEC) printf a
#endif

#ifdef WDC_DEBUG
#define	WDC_PRINTF(b, a)	if((b & wdc_debug_mask) != 0) printf a
#else
#define	WDC_PRINTF(b, a)
#endif

#define	WDC_DELAY_DEBUG		80
#define	WDC_DELAY		1
#define	WDC_TIMEOUT		(6000)

/*
 *	wdc_delay:
 *
 *	A simple delay routine.
 */
void wdc_delay(
	int	cnt)
{
	int i;
	
	for(i = 0; i < (cnt * 100); i++)
		/*asm("not"); */;
}

/*
 *	wdc_wait:
 *
 *	XXX
 */
int wdc_wait(
	wdc_softc_t		sc,
	u_int8_t		mask,
	u_int8_t		bits,
	int			timeout)	/* in 10ms */
{
	wdc_status_t value;
	u_int8_t error;
	int time = 0;

#if 0
	WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_wait: %s, mask = %#x, bits = %#x\n", 
		sc->sc_dev->dv_name, mask, bits));	
#endif

	for(;;) {
		value = isa_read_1(sc->sc_cmdtag,
				WDC_REGISTER_STATUS);
		if((value & WDC_STATUS_BUSY) == 0 && ((value & mask) == bits)) 
			break;
		if(time++ > timeout) {
			error = isa_read_1(sc->sc_cmdtag,
					WDC_REGISTER_ERROR);
			WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_wait: timeout, value = %#x, error = %#x\n", 
				value, error));
			return -1;
		}
#if 0
		wdc_delay(WDC_DELAY);
#endif
	}

	if(value & WDC_STATUS_ERROR) {
		error = isa_read_1(sc->sc_cmdtag,
				WDC_REGISTER_ERROR);
		WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_wait: masked bits ok, but error = %#x\n",
				error));
	}
#ifdef WDC_DELAY_DEBUG
	if(time > WDC_DELAY_DEBUG) {
		printf("%s channel %d drive %d: warning: busy-wait took more than %d loops\n",
			sc->sc_dev->dv_name, sc->sc_channel, sc->sc_drive, time);
	}
#endif
	return 0;
}

/*
 *	wdc_error:
 *
 *	Return an error value. If no error, the routine returns
 *	zero.
 */
int wdc_error(
	wdc_softc_t sc)
{
	u_int8_t stat = isa_read_1(sc->sc_cmdtag,
				WDC_REGISTER_STATUS);
	u_int8_t mask = WDC_STATUS_BUSY 
			| WDC_STATUS_READY 
			| WDC_STATUS_WRERR 
			| WDC_STATUS_SEEK
			| WDC_STATUS_ERROR;
	
	if((stat & mask) == (WDC_STATUS_READY | WDC_STATUS_SEEK))
		return 0;
	return isa_read_1(sc->sc_cmdtag,
			WDC_REGISTER_ERROR);
}

/*
 *	wdc_ready_controler:
 *
 *	Checks if the controler is ready.
 */
int wdc_ready_controler(
	isa_tag_t	tag,
	int		timeout)
{
	unsigned char value, mask;
	int retries = 1;
	
	mask	= WDC_STATUS_READY | WDC_STATUS_BUSY;

	do {
	again:
		value = isa_read_1(tag,
				WDC_REGISTER_STATUS);	
		if((value & mask) == WDC_STATUS_READY)
			break;
		if(timeout == 0)
			goto again;
	} while(retries++ < timeout);
	return retries;
}

/*
 *	wdc_ready_data:
 *
 *	Waits for data request.
 */
int wdc_ready_data(
	isa_tag_t	tag,
	int		timeout)
{
	unsigned char value, mask, old = 0;
	int retries = 1;
	
	mask	= WDC_STATUS_READY | WDC_STATUS_BUSY | WDC_STATUS_DRQ;

	do {
	again:
		value = isa_read_1(tag,
				WDC_REGISTER_STATUS);	
#if 1
		if(value != old) {
			printf("wdc_ready_data: value = %#x\n", value);
			old = value;
		}
#endif
		if((value & mask) == (WDC_STATUS_READY | WDC_STATUS_DRQ)) {
			printf("wdc_ready_data: return: value = %#x\n", value);
			return 1;
		}
		if(timeout == 0)
			goto again;
	} while(retries++ < timeout);
	return 0;
}

/*
 *	wdc_ready_reset:
 */
int wdc_ready_reset(
	isa_tag_t	tag,
	int		drvmask,
	int		timeout)
{
	int st0, st1;
	int time;

	WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_ready_reset: drvmask = %#x\n", drvmask));
	
	for(time = 0; drvmask ; time++) {
		/* master */
		isa_write_1(tag, WDC_REGISTER_DRIVE, WDC_ADDR_IBM);
		wdc_delay(20);
		st0 = isa_read_1(tag, WDC_REGISTER_STATUS);

		/* slave */
		isa_write_1(tag, WDC_REGISTER_DRIVE, WDC_ADDR_IBM | 0x10);
		wdc_delay(20);
		st1 = isa_read_1(tag, WDC_REGISTER_STATUS);
	
		WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_ready_reset: st0 = %#x, st1 = %#x\n", 
						st0, st1));

		if(st0 == 0xff)
			drvmask &= ~1;
		if(st1 == 0xff)
			drvmask &= ~2;

		if((drvmask & 1) == 0) {
			if((drvmask & 2) != 0 && (st1 & WDC_STATUS_BUSY) == 0)
				goto out;
		}
		else if((drvmask & 2) == 0) {
			if((drvmask & 1) != 0 && (st0 & WDC_STATUS_BUSY) == 0)
				goto out;
		}
		else {
			if((st0 & WDC_STATUS_BUSY) == 0 && (st1 & WDC_STATUS_BUSY) == 0)
				goto out;
		}
					
		/* hr skall det testas lite tror jag */	
		if(time >= timeout)
			break;
	}
	/* timeout */
	WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_ready_reset: timeout\n"));
out:
	if((st0 & WDC_STATUS_BUSY && st0 & WDC_STATUS_READY) || st0 == 0 || st0 == 0xff)
		drvmask &= ~1;
	if((st1 & WDC_STATUS_BUSY && st1 & WDC_STATUS_READY) || st1 == 0 || st1 == 0xff)
		drvmask &= ~2;	
	return drvmask;
}

/*
 *	wdc_trigger_reset:
 *
 *	Tries to trigger a reset on the controler.
 */
#define  WDC_CTL_4BIT	0x08   /* use four head bits (wd1003) */
#define  WDC_CTL_RST	0x04   /* reset the controller */
#define  WDC_CTL_IDS	0x02   /* disable controller interrupts */

int wdc_trigger_reset(
	isa_tag_t	cmd,
	isa_tag_t	aux,
	int		drvmask,
	int		timeout)
{
	int rval = 0x3;
	
	/* assert SRST, wait for reset to complete */
	WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_trigger_reset: asserting SRST\n"));

	isa_write_1(cmd, WDC_REGISTER_DRIVE, WDC_ADDR_IBM);
	wdc_delay(10);
	isa_write_1(aux, WDC_REGISTER_AUX,
		WDC_CTL_RST | WDC_CTL_IDS);
	wdc_delay(10);
	isa_write_1(aux, WDC_REGISTER_AUX,
		WDC_CTL_IDS);
	wdc_delay(10);
	(void) isa_read_1(cmd, WDC_REGISTER_ERROR);
	isa_write_1(aux, WDC_REGISTER_AUX,
		WDC_CTL_4BIT);
	
	/* wait for reset */
	WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_trigger_reset: waiting for reset\n"));
	return wdc_ready_reset(cmd, drvmask, timeout);	
}

/*
 *	wdc_execute:
 *
 *	Execute a controler command.
 */
int wdc_execute(
	wdc_softc_t	sc,
	u_int8_t	command,
	u_int16_t	sectors,
	u_int16_t	cylinder,
	u_int16_t	head,
	u_int16_t	sector,
	int		timeout)
{
	WDC_PRINTF(WDCDEBUG_EXEC, ("wdc_execute: executing command %#2.2x:%d on drive %d (chs=%d/%d/%d)\n",
		command, sectors, sc->sc_drive, cylinder, head, sector));

	isa_write_1(sc->sc_auxtag,
		0, 0);
	isa_write_1(sc->sc_cmdtag,
		WDC_REGISTER_PRECOMP, 0);
	isa_write_1(sc->sc_cmdtag,
		WDC_REGISTER_NSECTORS, sectors);
	isa_write_1(sc->sc_cmdtag,
		WDC_REGISTER_SECTOR, sector);
	isa_write_1(sc->sc_cmdtag,
		WDC_REGISTER_CYLINDER_LOW, cylinder & 0xff);
	isa_write_1(sc->sc_cmdtag,
		WDC_REGISTER_CYLINDER_HIGH, cylinder >> 8);
	isa_write_1(sc->sc_cmdtag,
		WDC_REGISTER_DRIVE, WDC_ADDR_IBM | (sc->sc_drive << 4) | head);
	isa_write_1(sc->sc_cmdtag,
		WDC_REGISTER_COMMAND, command);

	WDC_PRINTF(WDCDEBUG_EXEC, ("wdc_execute: (return)\n"));
	return 0;
}

/*
 *	wdc_lb2chs:
 *
 *	Convert from LBA to CHS addressing.
 */
void wdc_lba2chs(
	wdc_softc_t	sc,
	u_int32_t	sector,
	unsigned int	*out_cylinder,
	unsigned int	*out_head,
	unsigned int	*out_sector)
{
	u_int32_t	temp;
	
	*out_cylinder	= sector / (sc->sc_geo_head * sc->sc_geo_sector);
	temp		= sector % (sc->sc_geo_head * sc->sc_geo_sector);
	*out_head	= temp / sc->sc_geo_sector;
	*out_sector	= temp % sc->sc_geo_sector + 1;
}

/*
 *	wdc_xfer_allocate:
 *
 *	Allocate memory for a xfer buffer that can be used to
 *	access data on the drive.
 */
wdc_xfer_t wdc_xfer_allocate(
	wdc_softc_t sc)
{
	wdc_xfer_t xf;
	
	xf = (wdc_xfer_t) malloc(sizeof *xf);
	if(xf == WDC_XFER_NULL) {
		WDC_PRINTF(WDCDEBUG_XFER,
			("wdc_xfer_allocate: can't allocate memory for xfer buffer\n"));
		return WDC_XFER_NULL;
	}
	return xf;
}

/*
 *	wdc_xfer_setup:
 */
void wdc_xfer_setup(
	wdc_softc_t	sc,
	wdc_xfer_t	xf,
	wdc_command_t	command,	
	u_int32_t	sector,
	char		*buffer,
	u_int32_t	bcount)
{
	WDC_PRINTF(WDCDEBUG_XFER, 
		("wdc_xfer_setup called\n"));

	/** initialize structure: */
	xf->xf_command	= command;
	xf->xf_sector	= sector;
	xf->xf_buffer	= buffer;
	xf->xf_bcount	= bcount;
	
	/** clear fields: */
	xf->xf_error	= 0;
	xf->xf_status	= 0;
	xf->xf_flags	= 0;
}

/*
 *	wdc_xfer_start:
 */
int wdc_xfer_start(
	wdc_softc_t	sc,
	wdc_xfer_t	xf)
{
	int rval, cnt;

	WDC_PRINTF(WDCDEBUG_XFER,
		("wdc_xfer_start: xf = %#x\n", xf));

	/*
	 *	First we convert from LBA to CHS.
	 */
	if((xf->xf_flags & WDC_XFER_FLAG_NOSECTOR) == 0) {
		wdc_lba2chs(sc, 
			xf->xf_sector,
			(unsigned int *) &xf->xf_reg_cyl,
			(unsigned int *) &xf->xf_reg_head, 
			(unsigned int *) &xf->xf_reg_sect);	
		WDC_PRINTF(WDCDEBUG_XFER,
			("wdc_xfer_start: sector %d: cyl = %d, head = %d, sect = %d\n",
			xf->xf_sector, xf->xf_reg_cyl, 
			xf->xf_reg_head, xf->xf_reg_sect));
	}
	else {
		xf->xf_reg_cyl =
		xf->xf_reg_head =
		xf->xf_reg_sect = 0;
	}

	/*
	 * Must select drive here ?
	 */
	isa_write_1(sc->sc_cmdtag, WDC_REGISTER_DRIVE,
		WDC_ADDR_IBM | (sc->sc_drive << 4));

	/*
	 *	Wait.	
	 */
	if(wdc_wait(sc, sc->sc_rdy_mask, sc->sc_rdy_bits, 60)) 
		return 1;
	WDC_PRINTF(WDCDEBUG_XFER, ("wdc_xfer_start: after wdc_wait()\n"));

#ifdef ENABLE_MULTI
	/*
	 *	Check for multiread.
	 */
	if(xf->xf_bcount > 512 && xf->xf_command == WDC_COMMAND_READ) {
		xf->xf_command = WDC_COMMAND_READMULTI;
#if 0
		WDC_PRINTF(WDCDEBUG_XFER, ("wdx_xfer_start: switching to multiple mode\n"));

	
		/*
		 * Set multiple mode!
		 */
		rval = wdc_execute(sc,
			0xc6,			/* WDC_COMMAND_SETMULTI */
			xf->xf_bcount / 512,	/* # sectors */
			0,			/* cylinder */
			0,			/* head */
			0,			/* sector */
			6000);			/* timeout */
		if(rval != 0) {
			printf("%s channel %d drive %d: can't set multiple mode for reading %d sectors, starting at %d\n",
				sc->sc_dev->dv_name, sc->sc_channel, sc->sc_drive,
				xf->xf_bcount/512, xf->xf_sector);
			return rval;
		}
#endif
	}
#endif

	/*
	 *	Execute the command.
	 */
	rval = wdc_execute(sc,
		xf->xf_command, 	/* command */
		xf->xf_bcount / 512,	/* # sectors */
		xf->xf_reg_cyl,		/* cylinder */
		xf->xf_reg_head,	/* head */
		xf->xf_reg_sect,	/* sector */
		6000);			/* timeout */
	
#if 0
	if(wdc_wait(sc, sc->sc_drq_mask, sc->sc_drq_bits, 6000)) {
		xf->xf_status	= isa_read_1(sc->sc_cmdtag,
					WDC_REGISTER_STATUS);
		xf->xf_error	= isa_read_1(sc->sc_cmdtag,
					WDC_REGISTER_ERROR);
		xf->xf_flags |= WDC_XFER_FLAG_ERROR;
		return 1;
	}
#endif
	return wdc_intr(0, sc, xf);
}

/*
 *	wdc_read:
 *
 *	Issues a read command on the specifed sector.
 */
int wdc_read(
	wdc_softc_t	sc,
	int		sector,
	int		num,
	char		*buffer)
{
	wdc_xfer_t xf;
	
	xf = (wdc_xfer_t) wdc_xfer_allocate(sc);
	if(xf == WDC_XFER_NULL)
		return 1;
	
	(void) wdc_xfer_setup(sc, xf,
			(num > 1 ? WDC_COMMAND_READ 
				: WDC_COMMAND_READ),
			sector,
			buffer,
			num * 512);
	return wdc_xfer_start(sc, xf);
}


/*
 *	wdc_read_mbr_parition:
 *
 *	Read partitions.
 */
int wdc_read_mbr_partitions(
	wdc_softc_t	sc)
{
	struct partition {
		u_int8_t	p_flags;
		u_int8_t	p_chs_start[3];
		u_int8_t	p_type;
		u_int8_t	p_chs_end[3];
		u_int32_t	p_lba_start;
		u_int32_t	p_lba_size;
	} *p;
#define	PARTITION_OFF	0x1be
	int i, found = 0;
	
	if(wdc_read(sc, 0, 1, sc->sc_dbuff) != 0) {
		printf("%s: couldn't read partition table\n",
			sc->sc_dev->dv_name);
		return 1;
	}
	
	p = (struct partition *) (sc->sc_dbuff + PARTITION_OFF);
	if(sc->sc_dbuff[510] != 0x55 || (sc->sc_dbuff[511] & 0xff) != 0xaa) {
		printf("%s: %x, %x\n", sc->sc_dev->dv_name,
			sc->sc_dbuff[510], sc->sc_dbuff[511]);

		printf("%s: drive contains bad partition table\n",
			sc->sc_dev->dv_name);
		return 0;
	}
	
	/*
	 *	Loop through all partitions.
	 */
	printf("%s: partition table:\n", sc->sc_dev->dv_name);
	for(i = 0; i < 4; i++, p++) {
		wdc_partition_t wdp = &sc->sc_partition[i];

		/*
		 *	Make the partition unvalid for now.
		 */
		wdp->wdp_valid	= 0;

		/*
		 *	Copy information.
		 */
		wdp->wdp_start	= p->p_lba_start;
		wdp->wdp_length	= p->p_lba_size;
		wdp->wdp_type	= p->p_type;

		if(wdp->wdp_start == 0)
			continue;
		wdp->wdp_valid	= 1;

		printf("\t%s:%d: %8.8x (%8.8x sectors) type: %x%s\n",
			sc->sc_dev->dv_name, i, 
			p->p_lba_start, p->p_lba_size, p->p_type,
			(p->p_flags & 0x80 ? " (active)" : ""));

		if(p->p_type == 0xa9) 
			found = wdc_read_bsd_partitions(sc, &sc->sc_partition[i]); 
	}
	if(found == 0) {
		printf("%s: couldn't find any bsd slices on this disk.\n",	
			sc->sc_dev->dv_name);
	}
	return 0;
}

/*
 *	wdc_read_bsd_partitions:
 *
 *	Reads *BSD partitions (slices).
 */
#define	BSD_PART_SECTOR		1
#define	BSD_PART_MAGIC		0x82564557

#define	BSD_OFFSET_MAGIC	0
#define	BSD_OFFSET_PART		148
#define	BSD_OFFSET_NPART	138
 
#define	u_int32_cast(d, o)	( *( (u_int32_t *) ( ( (char*) (d) ) + (o) ) ) )
#define	u_int16_cast(d, o)	( *( (u_int16_t *) ( ( (char*) (d) ) + (o) ) ) )

/*
 *	BSD slice information:
 */
typedef struct wdc_slice {
	u_int32_t	wds_length;
	u_int32_t	wds_start;
	u_int32_t	wds_fragsize;
	u_int8_t	wds_type;
	u_int8_t	wds_fragperblock;
	u_int8_t	wds_extra;
	u_int8_t	__wds_pad;		/* pad */
} *wdc_slice_t;

 
int wdc_read_bsd_partitions(
	wdc_softc_t	sc,
	wdc_partition_t	wdp)
{
	char *buff = sc->sc_dbuff;
	u_int32_t *magic;
	u_int16_t *nparts;
	int partno;

	/*
	 *	Type of slices:
	 */
	static const char *types[] = {
		"unknown type", "swap", "Sixth Edition", "Seventh Edition",
		"System V", "V7 with 1K blocks", "Eighth Edition, 4K blocks",
		"4.2BSD fast file system", "MSDOS file system", 
		"4.4 log-structured file system", "unknown (but in use)",
		"OS/2 high-performance file system", "ISO 9660",
		"bootstrap"
	};

	/*
	 *	Read the sector that contain the BSD slices.
	 */
	if(wdc_read(sc, wdp->wdp_start + BSD_PART_SECTOR, 1, sc->sc_dbuff) != 0) {
		printf("%s: couldn't read slices\n",
			sc->sc_dev->dv_name);
		return 0;
	}
	
	printf("%s: bsd slices:\n", sc->sc_dev->dv_name);

	/*
	 *	Get the magic value and number of partitions.
	 */
	magic	= (u_int32_t *) ((char *) sc->sc_dbuff + BSD_OFFSET_MAGIC);
	nparts 	= (u_int16_t *) ((char *) sc->sc_dbuff + BSD_OFFSET_NPART);

	if(*magic == BSD_PART_MAGIC) {
		wdc_slice_t	wds = (wdc_slice_t) (sc->sc_dbuff + BSD_OFFSET_PART);
		wdc_partition_t	wdp;
		
		for(partno = 0; partno < *nparts; partno++) {
			wdp = (wdc_partition_t) &sc->sc_partition[NPART + partno];
		
			/*
			 *	Make the slice unvalid for now.
			 */
			wdp->wdp_valid = 0;

			/*
			 *	Copy slice information.
			 */
			wdp->wdp_start		= wds->wds_start;
			wdp->wdp_length		= wds->wds_length;
			wdp->wdp_type		= wds->wds_type;
			wdp->wdp_extra		= wds->wds_extra;
			wdp->wdp_fragsize	= wds->wds_fragsize;
			wdp->wdp_fragperblock	= wds->wds_fragperblock;
			
			/*
			 *	Check for valid slice:
			 */
			if(wdp->wdp_start != 0 && wdp->wdp_type != 0) {
				wdp->wdp_valid = 1;
				
				printf("\t%s%c: %8.8x (%8.8x sectors), %dMB, %s\n",
					sc->sc_dev->dv_name, ('a' + partno),
					wds->wds_start, wds->wds_length, (wds->wds_length * 512) >> 20,
					types[wds->wds_type]);
			}
			wds++;
		}
		return 1;
	}
	return 0;
}

/* 
 *	wdc_geometry:
 *
 *	Probes the drive for geometry and other information.
 */
int wdc_ata_params(
	wdc_softc_t	sc)
{
	u_int16_t dinfo[256];
	wdc_xfer_t xf;
	int cnt, rval;

	xf = wdc_xfer_allocate(sc);
	if(xf == WDC_XFER_NULL)
		return 1;
	
#if 0
	(void) wdc_xfer_setup(sc, xf,
			0xec,
			0,
			(char *) dinfo,
			512);

	xf->xf_flags |= WDC_XFER_FLAG_NOSECTOR;
	
	rval = wdc_xfer_start(sc, xf);
	if(rval == 0) {
		sc->sc_geo_cylinder	= dinfo[1];
		sc->sc_geo_head		= dinfo[3];
		sc->sc_geo_sector	= dinfo[6];

		printf("%s: geometry information: cylinders = %d, heads = %d, sectors = %d\n",
			sc->sc_dev->dv_name, dinfo[1], dinfo[3], dinfo[6]);
	}
	else {
		printf("%s: geometry: controler not availible\n",
			sc->sc_dev->dv_name);
		return 1;
	}
#else
	(void) wdc_xfer_setup(sc, xf,
			0xec,
			0,
			(char *) &sc->sc_ata_params,
			512);

	xf->xf_flags |= WDC_XFER_FLAG_NOSECTOR;
	
	rval = wdc_xfer_start(sc, xf);
	return rval;
#endif
	return 0;
}

/*
 *	wdc_probe:
 *
 *	Probe the controler to see if there's any disks
 *	on it.
 *
 *	The attach argument is the same that will be sent
 *	to the attach routine.
 *
 *	This is a generic routine that is not used in the
 *	driver.
 */
int wdc_probe(
	pciide_attach_args_t	pi,		/* attach argument */
	int			timeout)	/* timeout */
{
	u_int8_t st0, st1;
	int ret = 3;
	
	if(wdc_ready_controler(pi->pi_cmdtag, timeout) == 0) {
		printf("wd%d (%d:%d): controler not ready (timeout)\n",
			pi->pi_bargs.ba_unit, pi->pi_channel, pi->pi_drive);
		return 0;
	}

	(void) isa_write_1(pi->pi_cmdtag, 
			WDC_REGISTER_DRIVE, WDC_ADDR_IBM);
	wdc_delay(10);
	st0 = isa_read_1(pi->pi_cmdtag,
			WDC_REGISTER_STATUS);
	(void) isa_write_1(pi->pi_cmdtag,
			WDC_REGISTER_DRIVE, WDC_ADDR_IBM | 0x10);
	wdc_delay(10);
	st1 = isa_read_1(pi->pi_cmdtag,
			WDC_REGISTER_STATUS);
	if(st0 == 0xff)
		ret &= ~0x1;
	if(st1 == 0xff)
		ret &= ~0x2;
	if(ret == 0)
		return ret;
		
	/*
	 * Trigger a reset.
	 */
	ret = wdc_trigger_reset(pi->pi_cmdtag, pi->pi_auxtag, ret, 60);
#if 0
	printf("wdc_probe: ret = %x\n", ret);
#endif
	return ret;
}

/*
 *	wdc_intr:
 */
int wdc_intr(
	int		irq,
	wdc_softc_t	sc,
	wdc_xfer_t	xf)
{
	int cnt = 0;

	WDC_PRINTF(WDCDEBUG_ALL, ("wdc_intr: %s channel %d drive %d\n",
		sc->sc_dev->dv_name, sc->sc_channel, sc->sc_drive));
	
#ifndef ENABLE_MULTI
	for(cnt = 0; cnt < xf->xf_bcount / 512; cnt++) {
		if(wdc_wait(sc, sc->sc_drq_mask, sc->sc_drq_bits, WDC_TIMEOUT)) {	
			WDC_PRINTF(WDCDEBUG_ALL, ("wdc_intr: timeout\n"));
			return 1;
		}

		isa_read_multi_2(sc->sc_cmdtag,
			WDC_REGISTER_DATA,
			xf->xf_buffer + (cnt * 512),
			512 >> 1);

#ifdef WDC_DEBUG
		printf("wdc_intr: %d: status = %#x, error = %#x\n", cnt,
			isa_read_1(sc->sc_cmdtag, WDC_REGISTER_STATUS),
			isa_read_1(sc->sc_cmdtag, WDC_REGISTER_ERROR));

#endif
	}
#else

	WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_intr: entring wait\n"));
	if(wdc_wait(sc, sc->sc_drq_mask, sc->sc_drq_bits, WDC_TIMEOUT)) {	
		WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_intr: timeout\n"));
		return 1;
	}
	WDC_PRINTF(WDCDEBUG_WAIT, ("wdc_intr: (exit)\n"));

#if 0
	printf("wdc_intr: %d: status = %#x, error = %#x\n", cnt,
		isa_read_1(sc->sc_cmdtag, WDC_REGISTER_STATUS),
		isa_read_1(sc->sc_cmdtag, WDC_REGISTER_ERROR));
#endif

	isa_read_multi_2(sc->sc_cmdtag,
		WDC_REGISTER_DATA,
		xf->xf_buffer,
		xf->xf_bcount >> 1);

#if 0
	printf("wdc_intr: %d: status = %#x, error = %#x\n", cnt,
		isa_read_1(sc->sc_cmdtag, WDC_REGISTER_STATUS),
		isa_read_1(sc->sc_cmdtag, WDC_REGISTER_ERROR));
#endif
#endif
	return 0;
}

 
/*
 *	wdc_match:
 */
int wdc_match(
	device_t		parent,
	cfdata_t		cf,
	pciide_attach_args_t	pi)
{
	extern struct cfdriver cd_pci_pciide;
	int rval;

	if(parent->dv_cfdata->cf_driver != &cd_pci_pciide)
		return CF_MATCH_NONE;

#if 0
	rval = wdc_probe(pi, 12000);
	if(rval & (1 << pi->pi_drive))
#endif
		return CF_MATCH_EXACT;
#if 0
	return CF_MATCH_NONE;
#endif
}

/*
 *
 */
char* wdc_hdr(
	wdc_softc_t sc)
{
	static char buff[128];

	sprintf(buff, "%s: channel %d drive %d",
		sc->sc_dev->dv_name, 
		sc->sc_channel,
		sc->sc_drive);
	return buff;
}

/*
 *	wdc_attach:
 */
int wdc_attach(
	device_t		parent,
	device_t		self,
	pciide_attach_args_t	pi)
{
	wdc_softc_t sc = (wdc_softc_t) self->dv_softc;
	extern struct device_ops wdc_ops;
	int rval;

	printf("%s on %s channel %d drive %d\n",
		self->dv_name, parent->dv_name,
		pi->pi_channel, pi->pi_drive);

	/*
	 *	Set debug things.
	 */
	wdc_debug_mask = WDCDEBUG_EXEC
			| WDCDEBUG_WAIT;

	/*
	 *	Set device operations.
	 */
	self->dv_ops	= &wdc_ops;

	/*
	 *	Copy some information.
	 */
	sc->sc_dev	= self;
	 
	sc->sc_cmdtag	= pi->pi_cmdtag;
	sc->sc_auxtag	= pi->pi_auxtag;
	
	sc->sc_channel	= pi->pi_channel;
	sc->sc_drive	= pi->pi_drive;

	/*
	 *	Set flags:
	 */
	sc->sc_flags	= WDC_DRIVE_FLAG_POLL;

	/*
	 *	Initialize masks and bits.
	 */
#if 0
	sc->sc_drq_mask	= WDC_STATUS_READY | 
			WDC_STATUS_DRQ;
	sc->sc_drq_bits	= WDC_STATUS_READY | 
			WDC_STATUS_DRQ;
#else
	sc->sc_drq_mask	= WDC_STATUS_DRQ;
	sc->sc_drq_bits	= WDC_STATUS_DRQ;
#endif

#if 0	
	sc->sc_rdy_mask	= WDC_STATUS_READY |
			WDC_STATUS_SEEK;
	sc->sc_rdy_bits	= WDC_STATUS_READY |
			WDC_STATUS_SEEK;
#else
	sc->sc_rdy_mask	= WDC_STATUS_READY;
	sc->sc_rdy_bits	= WDC_STATUS_READY;
#endif

	/*
	 *	Allocate a internal sector buffer.
	 */
	sc->sc_dbuff	= (char *) malloc(512);

	/*
	 *	Get the geometry information from the drive.
	 */
	rval = wdc_ata_params(sc);
	if(rval != 0) {
		printf("%s: channel %d drive %d: no geometry information\n",
			self->dv_name,
			pi->pi_channel, pi->pi_drive);
		return 0;
	}
	else {
		sc->sc_geo_cylinder	= sc->sc_ata_params.atap_cylinders;
		sc->sc_geo_head		= sc->sc_ata_params.atap_heads;
		sc->sc_geo_sector	= sc->sc_ata_params.atap_sectors;
		
		printf("%s: geometry: cylinders = %d, heads = %d, sectors = %d\n",
			sc->sc_dev->dv_name, sc->sc_geo_cylinder, sc->sc_geo_head, sc->sc_geo_sector);
		
		/*
		 * Check if we can read more than one sector at time.
		 */
		if((sc->sc_ata_params.atap_multi & 0xff) > 1) {
			sc->sc_multi = sc->sc_ata_params.atap_multi & 0xff;

#ifdef ENABLE_MULTI
			printf("%s: info: seting multiple sector mode to # sectors: %d\n", 
				sc->sc_dev->dv_name, sc->sc_multi);

			/*
			 * Set multiple mode!
			 */
			rval = wdc_execute(sc,
				0xc6,			/* WDC_COMMAND_SETMULTI */
				sc->sc_multi,		/* # sectors */
				0,			/* cylinder */
				0,			/* head */
				0,			/* sector */
				6000);			/* timeout */
			if(rval != 0) {
				printf("%s: channel %d drive %d: can't set multiple mode for reading %d sectors\n",
					sc->sc_dev->dv_name, sc->sc_channel, sc->sc_drive, sc->sc_multi);
			}
#endif
		}
		else {
			sc->sc_multi = 1;
		}
		
	}
	
	/*
	 *	Read partition table here:
	 */
	wdc_read_mbr_partitions(sc);
	return 0;
}

/*
 *	Standrad I/O routines.
 */
#define	RAWPART		-1

device_unit_t wdc_io_open(
	device_unit_t	du)
{
	wdc_softc_t sc = (wdc_softc_t) du->du_device->dv_softc;
	int unit = 0;

	/*
	 * 	Check if we want to open the raw disk.
	 */
	if(du->du_unit == RAWPART)
		return du;
	
	/*
	 *	Check so that we don't tries to open a 
	 *	partition out of range.
	 */
	if(du->du_unit >= 16 && du->du_unit < (16 + NPART)) {	/* mbr partition */
		unit = du->du_unit - 16;
		
		if(sc->sc_partition[unit].wdp_valid == 0)
			return DEVICE_UNIT_NULL;
	
		du->du_data = (void *) &sc->sc_partition[unit];
		return du;
	}
	
	/*
	 *	Check for a valid slice.
	 */
	if(du->du_unit < NSLICE) {
		if(sc->sc_partition[NPART + du->du_unit].wdp_valid == 0)
			return DEVICE_UNIT_NULL;
		du->du_data = (void *) &sc->sc_partition[NPART + du->du_unit];
		return du;
	}
	return DEVICE_UNIT_NULL;
}

/*
 *	Block I/O routines that are specific for the wd device:
 */
int wdc_blk_read(
	device_unit_t	du,
	u_int32_t	sector,
	u_int32_t	nsectors,
	char		*buffer)
{
	wdc_softc_t sc = (wdc_softc_t) du->du_device->dv_softc;
	u_int32_t offset = 0, unit;


	if(du->du_unit >= 16 && du->du_unit < (16 + NPART)) {	/* mbr partition */
		unit = du->du_unit - 16;
		
		if(sc->sc_partition[unit].wdp_valid == 0)
			-1;
	
		offset = sc->sc_partition[unit].wdp_start;
	}
	
	/*
	 *	Check for a valid slice.
	 */
	if(du->du_unit < NSLICE) {
		if(sc->sc_partition[NPART + du->du_unit].wdp_valid == 0)
			-1;
		offset = sc->sc_partition[NPART + du->du_unit].wdp_start;
	}

	/*
	 *
	 */
#ifdef DEBUG
	printf("wdc_blk_read: tries to read %d sectors from %x\n",
			nsectors, offset + sector);
#endif

	/*
	 *	Do the reading here.
	 */
#if 1
	return wdc_read(sc, offset + sector, nsectors, buffer);
#else
	{
		int i = 0, ret; 

#ifdef DEBUG		
		printf("wdc_blk_read: nsectors = %d\n", nsectors);
#endif

		for(i = 0; i < nsectors; i++) {
			ret = wdc_read(sc, 
				offset + sector + i,
				1, buffer + (512 * i));
			if(ret != 0)
				break;
		}
		return ret;
	}
#endif
}

/* 
 *	Device operations:
 */
struct device_ops wdc_ops = {
	wdc_io_open		/* open */
};

/*
 *	Device configration data:
 */
struct cfattach ca_pciide_wd = {
	sizeof(struct wdc_softc),
	wdc_match,	/* match */
	wdc_attach,	/* attach */
	0,		/* detach */
	"wd"		/* name */
};
