/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: ffs_fsa.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: ffs_fsa.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <rom/fs.h>
#include <rom/file.h>
#include "ffs.h"

#if 0
#define	DEBUG
#endif

#if 0
/* 
 *	ffs softc:
 */
typedef struct ffs_softc {
	device_unit_t	sc_unit;
	char *		sc_buff;
	struct ffs *	sc_super;
} *ffs_softc_t;
#endif

/*
 *	ffs_mount:
 *
 *	Implements the mount action.
 */
int ffs_mount(
	filesystem_action_t fsa)
{
	ffs_softc_t sc = (ffs_softc_t) fsa->fsa_softc;
	int ret;

	/*
	 *	Set our device.
	 */
	sc->sc_unit = fsa->fsa_unit;
	 

	/*
	 *	Allocate memory for the superblock.
	 */
	sc->sc_super = (struct ffs *) malloc(SBSIZE);
	if(sc->sc_super == 0) {
#ifdef DEBUG
		printf("ffs_mount: no memory\n");
#endif
		return -1;
	}
	
	/*
	 *	Read the superblock.
	 */
#if 0
	ret = wd_blk_read(sc->sc_unit, 
		SBLOCK, 
		(SBSIZE/DEV_BSIZE),
		sc->sc_super);
	if(ret != 0) {
		fre(sc->sc_super);
		printf("ffs_mount: no read super\n");
		return ret;	
	}
#else
	ret = device_blk_read(sc->sc_unit,
		SBLOCK,
		(SBSIZE/DEV_BSIZE),
		(char *) sc->sc_super);
	if(ret != 0) {
		free(sc->sc_super);
		printf("ffs_mount: no read super\n");
		return ret;	
	}
#endif

	/*
	 *	Check if it is a valid filesystem:
	 */
	if(sc->sc_super->fs_magic != FS_MAGIC) {
#ifdef DEBUG
		hexdump("block", ((char *) sc->sc_super), 1024*8);
		printf("ffs_mount: invalid magic (%#x = %#x)\n", 
			sc->sc_super->fs_magic,
			FS_MAGIC);
#endif	
		free(sc->sc_super);
		return 1;
	}

	/*
	 *	Okey. We got a filesystem here.
	 *
	 *	Allocate memory for the map buffer and the
	 *	common inode.
	 */
	sc->sc_mapbuf	= (int *) malloc(sc->sc_super->fs_bsize);
	sc->sc_inode	= (inode_t) malloc(sc->sc_super->fs_bsize);
	sc->sc_dbuf	= (char *) malloc(DEV_BSIZE * 32);

#ifdef DEBUG
	printf("ffs_mount: mount ok\n");
#endif
	return 0;
}

/*
 *
 */
int ffs_openfile(
	filesystem_action_t fsa)
{
	ffs_softc_t sc = (ffs_softc_t) fsa->fsa_softc;
	struct ffs_file *ffsf;
	struct file *file = (struct file *) fsa->fsa_file;
	int ino, rval;

#ifdef DEBUG
	printf("blksize : %d\n", sc->sc_super->fs_bsize);
#endif
	
	/*
	 * Check if the specified file really exist.
	 */
	rval = ffs_getino(sc, (char *) fsa->fsa_buffer, &ino);
	if(rval != 0) 
		return 1;
	
	/*
	 * Allocate structure.
	 */
	ffsf = (struct ffs_file *) malloc(sizeof *ffsf);
	if(ffsf == 0)
		return 2;
	ffsf->ffsf_bdata = (char *) malloc(sc->sc_super->fs_bsize);
	ffsf->ffsf_blk   = (u_int32_t) -1;
	
	/*
	 * Read the inode.
	 */
	rval = ffs_readino(sc, ino, ffsf);
	if(rval != 0)
		return 3;
	
	/*
	 * Set private data.
	 */
	file->f_aux = (void *) ffsf;
	return 0;
}

#if 0
int ffs_readbuff(
        ffs_softc_t     sc,     /* softc */
                struct ffs_file *file,  /* ffs file structure */
                        u_int32_t       offset, /* offset */
                                char *          buff,   /* buffer */
                                        u_int32_t       length) /* length */
#endif                                    
	

/*
 *	ffs_readfile:
 *
 *	Implements the ``FSA_READFILE action.
 */
int ffs_readfile(
	filesystem_action_t fsa)
{
	struct file *file = (struct file *) fsa->fsa_file;
	struct ffs_file *ffsf = (struct ffs_file *) file->f_aux;
	ffs_softc_t sc = (ffs_softc_t) fsa->fsa_softc;
	int rval;
	
	rval = ffs_readbuff(sc,
			ffsf,
			file->f_offset, 
			fsa->fsa_buffer,
			fsa->fsa_length);
	if(rval < 0)
		return -rval;
	file->f_offset += rval;
	fsa->fsa_actual = rval;
	return 0;
}

/*
 *	ffs_fsa:
 *
 *	This routine is the core of the FFS filesystem. It dispatches
 *	all requests from the I/O system.
 *
 *	Right now it implements the following actions:
 *		* mount, unmount
 *		* openfile, readfile
 */
int ffs_fsa(
	filesystem_action_t fsa)
{
	switch(fsa->fsa_action) {
		case FSA_MOUNT:
			fsa->fsa_error = ffs_mount(fsa);
			break;

		case FSA_UNMOUNT:
			break;

		case FSA_OPENFILE:
			fsa->fsa_error = ffs_openfile(fsa);
			break;

		case FSA_READFILE:
			fsa->fsa_error = ffs_readfile(fsa);
			break;
	}
	return fsa->fsa_error;
}

/*
 *	Filesystem attach data:
 */
struct filesystem_attach fa_ffs = {
	"ffs",				/* name of filesystem */
	sizeof(struct ffs_softc),	/* length of softc */
	ffs_fsa				/* action handler */
};
