/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: ffs_subr.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: ffs_subr.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include "ffs.h"

#if 0
#define	DEBUG
#endif

#ifndef DINODE_SIZE
#define	DINODE_SIZE	128
#endif

/*
 *	ffs_blockmap:
 */
int ffs_blockmap(
	ffs_softc_t	sc,	/* softc */
	inode_t		inode,	/* inode */
	u_int32_t	fblk,	/* file block */
	u_int32_t	*out)	/* out */
{
	static u_int32_t mapblk = -1;	/* block # in buffer */
	u_int32_t bnum;
	int rval;

	/*
	 * Check if the file block number is in the area of 
	 * direct addressing. If that is the case, we get the
	 * block number from the inode.
	 */
	if(fblk < NDADDR) {
		*out = inode->ino_db[fblk];
		return 0;
	}
	
	/* 
	 * If we have to use indirect addressing to get the 
	 * file block number, we first have to read the map
	 * bloc.
	 */
	if((bnum = fsbtodb(sc->sc_super, inode->ino_ib[0])) != mapblk) {
		rval = device_blk_read(sc->sc_unit,
				bnum,
				(sc->sc_super->fs_bsize / DEV_BSIZE),
				(char *) sc->sc_mapbuf);
		if(rval != 0) 
			return 1;

		mapblk = bnum;
	}	

	*out = sc->sc_mapbuf[(fblk - NDADDR) % NINDIR(sc->sc_super)];	
	return 0;
}

/*
 *	ffs_readbuff:
 *
 *	Reads from the specified file.
 */
int ffs_readbuff(
	ffs_softc_t	sc,	/* softc */
	struct ffs_file	*file,	/* ffs file structure */
	u_int32_t	offset,	/* offset */
	char *		buff,	/* buffer */
	u_int32_t	length)	/* length */
{
	inode_t inode = &file->ffsf_inode;
	u_int32_t location = offset;
	int rval, bread = 0, bsize;
	u_int32_t blk = -1, map;
	char *bdata = 0;

#ifdef DEBUG
	printf("ffs_readbuff: offset = %d, length = %d\n", offset, length);
#endif

	/*
	 * Here we do the reading.
	 */
	while(length != 0) {
		/*
		 * Check so that we don't exceed the length of the inode.
		 */
		if(location >= inode->ino_size) {
#ifdef DEBUG
			printf("inode size: %d\n", inode->ino_size);
#endif	
			return bread;
		}
		
		/*
		 * XXX 
		 */
		if(lblkno(sc->sc_super, location) != file->ffsf_blk) {
			file->ffsf_blk = lblkno(sc->sc_super, location);

			/*
			 * A test!
			 */
			if(length > blksize(sc->sc_super, inode, file->ffsf_blk)) {
				bdata = ((char *) buff) + bread;
				bsize = blksize(sc->sc_super, inode, file->ffsf_blk) - blkoff(sc->sc_super, location);
			}
			else {
				bdata = file->ffsf_bdata;	
			}

			/*
			 * Find block number for fileblock.
			 */
			rval = ffs_blockmap(sc, inode, file->ffsf_blk, &map);
			if(rval != 0) 
				return -11;
			rval = device_blk_read(sc->sc_unit,
					fsbtodb(sc->sc_super, map) , 
					blksize(sc->sc_super, inode, file->ffsf_blk) / DEV_BSIZE, 
					bdata);
			if(rval != 0)
				return -11;
		}

		if(bdata == 0 || bdata == file->ffsf_bdata) {
			/*
			 * Copy information.
			 */
			if(length > blksize(sc->sc_super, inode, file->ffsf_blk))
				bsize = blksize(sc->sc_super, inode, file->ffsf_blk) - blkoff(sc->sc_super, location);
			else
				bsize = length;
			Bcopy(((char *)file->ffsf_bdata) + blkoff(sc->sc_super, location), ((char *)buff) + bread, bsize);
		}

		/* 
		 * Step forward in the block.
		 */
		bread		+= bsize;
		length		-= bsize;
		location	+= bsize;

		progress();
 	}
#ifdef DEBUG
	printf("ffs_readbuff: (returning) bread = %d\n", bread);
#endif
 	return bread;
}

/*
 *	ffs_getino:
 *
 *	Reads a inode.
 *
 *	NOTE trashes the interal buffer.
 */
int ffs_readino(
	ffs_softc_t	sc,	/* softc */
	u_int32_t	ino,	/* inode number */
	struct ffs_file *file)	/* ffs file structure */
{
	inode_t inode;
	int rval;

	/*
	 * Read the node.
	 */
	rval = device_blk_read(sc->sc_unit,
			fsbtodb(sc->sc_super, ino_to_fsba(sc->sc_super, ino)),
			sc->sc_super->fs_bsize / DEV_BSIZE,
			(char *) sc->sc_inode);
	if(rval != 0) 
		return 1;

	/*
	 * We have now read the inode buffer into sc->sc_inode. Lets
	 * copy it into the ffs file structure.
	 */
	inode = (inode_t) ((char *) sc->sc_inode + (ino_to_fsbo(sc->sc_super, ino) * DINODE_SIZE));

#ifdef DEBUG
	printf("size: %d\n", inode->ino_size);
#endif
	bcopy(inode, &file->ffsf_inode, sizeof *inode);
#ifdef DEBUG
	printf("2 size: %d\n", file->ffsf_inode.ino_size);
#endif
	return 0;
}

/*
 *	ffs_getino:
 *
 *	Gets a inode number from a name.
 */
int ffs_getino(
	ffs_softc_t	sc,	/* softc */
	char		*name,	/* filename */
	u_int32_t	*out)	/* out */
{
	int ino = ROOTINO, rval;
	u_int32_t location = 0, offset = 0, blk, map, cnt = 0;
	char buf[128], *ptr, *s;
	struct inode *inode;
	struct direct *dp;
	
	/*
	 * Initialize the name pointer.
	 */
	ptr = name;
	
	while(1) {

		/*
		 * Get the name segment.
		 */
		for(s = ptr; *s && *s != '/'; s++)
			;
		strncpy(buf, ptr, ((int)s - (int)ptr));
		buf[((int)s - (int)ptr)] = '\0';

		ptr = (char *)s + 1;
		
		/*
		 * First of all we got to read the inode ofcouse.
		 */
		rval = device_blk_read(sc->sc_unit,
				fsbtodb(sc->sc_super, ino_to_fsba(sc->sc_super, ino)),
				sc->sc_super->fs_bsize / DEV_BSIZE,
				(char *) sc->sc_inode);
		if(rval != 0) 
			return 1;
			
		location = offset = 0;

		/*
		 * We have now read the inode buffer into sc->sc_inode. Lets
		 * get our inode.
		 */
		inode = (inode_t) ((char *) sc->sc_inode + (ino_to_fsbo(sc->sc_super, ino) * DINODE_SIZE));
		
		/*
		 * Sanity checks. If the loaded inode is not a directory we
		 * are far way from right.
		 *
		 * We also checks so the directory has a contents. 
		 */
		if((inode->ino_mode & IFMT) != IFDIR) 
			return 1;

		if(inode->ino_size == 0 || inode->ino_blocks == 0)
			return 1;

		/*
		 * Loop through the directory in search of the specified
		 * file/directory.
		 */
		do {
			/*
			 * Check so that we don't exceed the length of the inode.
			 */
			if(location >= inode->ino_size) {
				return 1;
			}
		
			/*
			 * If we are in the begining of a block we must ofcouse read
			 * the block into a buffer. 
			 */
			if(!(offset = blkoff(sc->sc_super, location))) {
				blk = lblkno(sc->sc_super, location);
		
				/*
				 * Find block number for fileblock.
				 */
				rval = ffs_blockmap(sc, inode, blk, &map);
				if(rval != 0) 
					return 1;
				rval = device_blk_read(sc->sc_unit,
						fsbtodb(sc->sc_super, map) , 
						blksize(sc->sc_super, inode, blk) / DEV_BSIZE, 
						sc->sc_dbuf);
				if(rval != 0)
					return 1;
			}
	
			/*
			 * Now the get the directory entry structure.  
			 */
			dp = (struct direct *) (sc->sc_dbuf + offset);
		
			/*
			 * Step forward in the block (to the next directory
			 * entry)
			 */
			location	= location + dp->d_reclen;

			/*
			 * Check if this directory entry is the one we are
			 * searching for. If that is the case, we just return.
			 */ 
			if(strcmp(buf, dp->d_name) == 0) {
				ino = dp->d_ino;
				break;
			}
 		} while(dp->d_ino);

		if(dp->d_ino == 0 && *s == '\0') {
			return 1;
		}

		if(*s == '\0' && dp->d_ino != 0) {
	 		*out = ino;
	 		return 0;
		}
	}
}
