/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: subr_fs.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: subr_fs.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <rom/fs.h>
#include <rom/file.h>

#if 0
#define	DEBUG
#endif

/** extern data: */
extern struct filesystem_attach fa_ffs;

/*
 *	Availible filesystems:
 */
filesystem_attach_t fs_filesystems[] = {
	&fa_ffs,	/* ffs */
	0
};

/*
 *	Our root filesystem;
 */
filesystem_t rootfs = FILESYSTEM_NULL;

/*
 */
filesystem_t filesystem_mount(
	char	*devname,
	int	unit)
{
	filesystem_t fs = FILESYSTEM_NULL;
	device_unit_t du = DEVICE_UNIT_NULL;
	struct filesystem_action fsa;
	filesystem_attach_t *fa;

#ifdef DEBUG
	printf("mount: dev = '%s', unit = %d\n", devname, unit);
#endif
	
	/*
	 *	First we try to open the device.
	 */
	du = (device_unit_t) device_open(devname, unit);
	if(du == DEVICE_UNIT_NULL) 
		return FILESYSTEM_NULL;
	
	/*
	 *	Allocate a filesystem structure:
	 */
	fs = (filesystem_t) malloc(sizeof *fs);
	if(fs == FILESYSTEM_NULL) {
		device_close(du);
		return FILESYSTEM_NULL;
	}

	/*
	 *	XXX comment here
	 */
	for(fa = fs_filesystems; *fa != 0; fa++) {
		fs->fs_fsah	= (*fa)->fa_fsah;

		fsa.fsa_unit	= du;
		fsa.fsa_action	= FSA_MOUNT;
		fsa.fsa_softc	= (char *) malloc((*fa)->fa_length);	

		if(filesystem_sendfsa(fs, &fsa) == 0) {
			fs->fs_aux = (void *) fsa.fsa_softc;
			return fs;
		}
	}
	return FILESYSTEM_NULL;
}

/*
 *
 *
 */
int filesystem_sendfsa(
	filesystem_t		fs,
	filesystem_action_t	fsa)
{
	return (*fs->fs_fsah)(fsa);
}

/*
 *	Array of files:
 *
 */
#ifndef NFILES
#define	NFILES	4
#endif

static struct file	fs_files[NFILES];
static int		fs_nfiles = 0;

/*
 *	open:
 *
 *	Standard I/O routine for opening files.
 */
int open(
	char	*name,
	int	mode)
{
	struct filesystem_action fsa;
	struct file *file;
	int rval, fd;
	
	/*
	 * Check so that we really have a filesystem
	 * mounted.
	 */
	if(rootfs == 0)
		return -1;

	/*
	 * Check so that we don't open to many files.
	 */
	if(fs_nfiles == NFILES)
		return -1;
	file = &fs_files[(fd = fs_nfiles++)];
	file->f_offset = 0;
	
	/* 
	 * Create a filesystem action and send it to the
	 * filesystem handler.
	 */
	fsa.fsa_action	= FSA_OPENFILE;
	fsa.fsa_buffer	= name;
	fsa.fsa_softc	= (char *) rootfs->fs_aux;
	fsa.fsa_file	= (void *) file;
	
	/** send it */
	rval = filesystem_sendfsa(rootfs, &fsa);
	if(rval != 0)
		return -2;
	return fd;
}

/* 
 *
 */
int read(
	int	fd,
	char	*buff,
	int	length)
{
	struct filesystem_action fsa;
	struct file *file;
	int rval;
	
	/*
	 * Check so that we really have a filesystem
	 * mounted.
	 */
	if(rootfs == 0)
		return -1;

	/*
	 * Check so that we don't try to read from an
	 * invalid file.
	 */
	if(fd >= fs_nfiles)
		return -1;
	
	/* 
	 * Create a filesystem action and send it to the
	 * filesystem handler.
	 */
	fsa.fsa_action	= FSA_READFILE;
	fsa.fsa_buffer	= buff;
	fsa.fsa_length	= length;
	fsa.fsa_softc	= (char *) rootfs->fs_aux;
	fsa.fsa_file	= (void *) &fs_files[fd];
	
	/** send it */
	rval = filesystem_sendfsa(rootfs, &fsa);
	if(rval != 0)
		return -2;
	if(fsa.fsa_error != 0)
		return -fsa.fsa_error; 
	return fsa.fsa_actual;
}

/*
 *
 */
int seek(
	int		fd,
	u_int32_t	offset)
{
	struct file *file;
	/*
	 * Check so that we really have a filesystem
	 * mounted.
	 */
	if(rootfs == 0)
		return -1;

	/*
	 * Check so that we don't try to read from an
	 * invalid file.
	 */
	if(fd >= fs_nfiles)
		return -1;
	
	/* 
	 * Set new position.
	 */
	file = &fs_files[fd];
	file->f_offset = offset;
	return 0;
}
