/*
 *  GNUBIOS
 *  Copyright 1999, 2000 Niklas Ekstrm
 *
 *  PCI Interface
 */

#ifndef GNUBIOS_PCI_H
#define GNUBIOS_PCI_H

/* PCI Configuration Space Entries */

#define PCI_VENDOR_ID								0x00
#define PCI_DEVICE_ID								0x02

#define PCI_COMMAND									0x04
	#define PCI_COMMAND_IO							0x0001
	#define PCI_COMMAND_MEMORY					0x0002
	#define PCI_COMMAND_MASTER					0x0004
	#define PCI_COMMAND_SPECIAL					0x0008
	#define PCI_COOMAND_MEMWAI					0x0010
	#define PCI_COMMAND_VGA_SNOOP				0x0020
	#define PCI_COMMAND_PARITY					0x0040
	#define PCI_COMMAND_WAIT						0x0080
	#define PCI_COMMAND_SERR						0x0100
	#define PCI_COMMAND_FAST_B2B				0x0200

#define PCI_STATUS									0x06
	#define PCI_STATUS_66_CAPABLE				0x0020
	#define PCI_STATUS_UDF							0x0040
	#define PCI_STATUS_FAST_B2B					0x0080
	#define PCI_STATUS_PARITY						0x0100
	#define PCI_STATUS_DEVSEL_MASK			0x0600
		#define PCI_STATUS_DEVSEL_FAST			0x0000
		#define PCI_STATUS_DEVSEL_MEDIUM		0x0200
		#define PCI_STATUS_DEVSEL_SLOW			0x0400
	#define PCI_STATUS_SIG_TARGET_ABORT	0x0800
	#define PCI_STATUS_RCV_TARGET_ABORT	0x1000
	#define PCI_STATUS_RCV_MASTER_ABORT	0x2000
	#define PCI_STATUS_SIG_SYSTEM_ERROR	0x4000
	#define PCI_STATUS_PARITY_ERROR			0x8000

#define PCI_CLASS_REVISION					0x08
	#define PCI_CLASS_MASK							0xffffff00
#define PCI_REVISION_ID							0x08
#define PCI_CLASS_PROG_IF						0x09
#define PCI_CLASS_SUB_CODE					0x0a
#define PCI_CLASS_BASE_CODE					0x0b

#define PCI_CACHE_LINE_SIZE					0x0c
#define PCI_LATENCY_TIMER						0x0d

#define PCI_HEADER_TYPE							0x0e
	#define PCI_HEADER_MULTIFUNC				0x80
	#define PCI_HEADER_TYPE_0						0x00
	#define PCI_HEADER_TYPE_P2P					0x01

#define PCI_BIST										0x0f
	#define PCI_BIST_COMPCODE_MASK			0x0f
	#define PCI_BIST_START							0x40
	#define PCI_BIST_CAPABLE						0x80

/* Type 00h Configuration Space Header */

#define PCI_BASE_ADDRESS_0					0x10
#define PCI_BASE_ADDRESS_1					0x14
#define PCI_BASE_ADDRESS_2					0x18
#define PCI_BASE_ADDRESS_3					0x1c
#define PCI_BASE_ADDRESS_4					0x20
#define PCI_BASE_ADDRESS_5					0x24
	#define PCI_ADDR_SPACE							0x01
		#define PCI_ADDR_SPACE_MEM					0x00
		#define PCI_ADDR_SPACE_IO						0x01
	#define PCI_ADDR_MEM_TYPE_MASK			0x06
		#define PCI_ADDR_MEM_TYPE_32				0x00
		#define PCI_ADDR_MEM_TYPE_1M				0x02
		#define PCI_ADDR_MEM_TYPE_64				0x04
	#define PCI_ADDR_MEM_PREFETCH				0x08

#define PCI_CARDBUS_CIS							0x28

#define PCI_SUBSYSTEM_VENDOR_ID			0x2c
#define PCI_SUBSYSTEM_ID						0x2e

#define PCI_ROM_BASE_ADDRESS				0x30
	#define PCI_ROM_BASE_ENABLED				0x01

#define PCI_INTERRUPT_LINE					0x3c
#define PCI_INTERRUPT_PIN						0x3d
#define PCI_MIN_GNT									0x3e
#define PCI_MAX_LAT									0x3f



/* Header type 1 (PCI-to-PCI bridges) */
#define PCI_PRIMARY_BUS		0x18	/* Primary bus number */
#define PCI_SECONDARY_BUS	0x19	/* Secondary bus number */
#define PCI_SUBORDINATE_BUS	0x1a	/* Highest bus number behind the bridge */
#define PCI_SEC_LATENCY_TIMER	0x1b	/* Latency timer for secondary interface */
#define PCI_IO_BASE		0x1c	/* I/O range behind the bridge */
#define PCI_IO_LIMIT		0x1d
#define  PCI_IO_RANGE_TYPE_MASK	0x0f	/* I/O bridging type */
#define  PCI_IO_RANGE_TYPE_16	0x00
#define  PCI_IO_RANGE_TYPE_32	0x01
#define  PCI_IO_RANGE_MASK	~0x0f
#define PCI_SEC_STATUS		0x1e	/* Secondary status register, only bit 14 used */
#define PCI_MEMORY_BASE		0x20	/* Memory range behind */
#define PCI_MEMORY_LIMIT	0x22
#define  PCI_MEMORY_RANGE_TYPE_MASK 0x0f
#define  PCI_MEMORY_RANGE_MASK	~0x0f
#define PCI_PREF_MEMORY_BASE	0x24	/* Prefetchable memory range behind */
#define PCI_PREF_MEMORY_LIMIT	0x26
#define  PCI_PREF_RANGE_TYPE_MASK 0x0f
#define  PCI_PREF_RANGE_TYPE_32	0x00
#define  PCI_PREF_RANGE_TYPE_64	0x01
#define  PCI_PREF_RANGE_MASK	~0x0f
#define PCI_PREF_BASE_UPPER32	0x28	/* Upper half of prefetchable memory range */
#define PCI_PREF_LIMIT_UPPER32	0x2c
#define PCI_IO_BASE_UPPER16	0x30	/* Upper half of I/O addresses */
#define PCI_IO_LIMIT_UPPER16	0x32
/* 0x34-0x3b is reserved */
#define PCI_ROM_ADDRESS1	0x38	/* Same as PCI_ROM_ADDRESS, but for htype 1 */
/* 0x3c-0x3d are same as for htype 0 */
#define PCI_BRIDGE_CONTROL	0x3e
#define  PCI_BRIDGE_CTL_PARITY	0x01	/* Enable parity detection on secondary interface */
#define  PCI_BRIDGE_CTL_SERR	0x02	/* The same for SERR forwarding */
#define  PCI_BRIDGE_CTL_NO_ISA	0x04	/* Disable bridging of ISA ports */
#define  PCI_BRIDGE_CTL_VGA	0x08	/* Forward VGA addresses */
#define  PCI_BRIDGE_CTL_MASTER_ABORT 0x20  /* Report master aborts */
#define  PCI_BRIDGE_CTL_BUS_RESET 0x40	/* Secondary bus reset */
#define  PCI_BRIDGE_CTL_FAST_BACK 0x80	/* Fast Back2Back enabled on secondary interface */

int		PciFindDevice( u8 bus, u32 venddev, int devfn );
int		PciFindClass( u8 bus, u32 devclass, int devfn);
int	PciAllocateResources();
#define DEVFN(dev, fn)  (((dev)<<3)|(fn))
#endif /* GNUBIOS_PCI_H */
