#ifndef BUS_H
#define	BUS_H

/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: bus.h,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#include <sys/cdefs.h>
#include <sys/types.h>

#include <rom/queue.h>
#include <rom/device.h>


/*
 *	Bus type:
 */
typedef int bus_type;

#define	BUS_TYPE_UNKNOWN	((bus_type) 0)	/* unknown bus type */
#define	BUS_TYPE_ISA		((bus_type) 1)	/* standard isa bus (I/O) */
#define	BUS_TYPE_PCI		((bus_type) 2)	/* standard pci bus */

/*
 *	Bus structure;
 */
typedef struct bus {
	bus_type		bus_type;	/* type of bus */
	u_int32_t		bus_number;	/* bus number */
	queue_head_t		bus_space;	/* I/O space */
	device_t		bus_device;	/* device */

	u_int32_t		bus_addr_base;	/* address space base */
	u_int32_t		bus_addr_limit;	/* address space limit */
} bus;
typedef bus			*bus_t;
#define	BUS_NULL		((bus_t) 0)
 
#define	BUS_ADDRESS_BASE_DEFAULT	(0)
#define	BUS_ADDRESS_LIMIT_DEFAULT	(0)
 
/*
 *	Bus space tag types:
 */
typedef int bus_space_tag_type;
 
#define	BUS_SPACE_TAG_TYPE_MEMORY	((bus_space_tag_type) 0)
#define	BUS_SPACE_TAG_TYPE_IO		((bus_space_tag_type) 1)
#define	BUS_SPACE_TAG_TYPE_PCI		((bus_space_tag_type) 2)	/* special tag */

/*
 *	Bus space tag structure:
 */
typedef struct bus_space_tag {
	bus_t			bst_bus;	/* bus that we conntected to */
	bus_space_tag_type	bst_type;	/* type of bus */
	unsigned long		bst_device;	/* device */
	unsigned long		bst_function;	/* function */
} bus_space_tag;
typedef bus_space_tag 		*bus_space_tag_t;
#define	BUS_SPACE_TAG_NULL	((bus_space_tag_t) 0)

/*
 *	Bus space handle structure:
 */
typedef struct bus_space_handle {
	queue_chain_t		bsh_node;	/* node in space */
	unsigned long		bsh_offset;	/* offset */
	unsigned int		bsh_length;	/* length */
} bus_space_handle;
typedef bus_space_handle	*bus_space_handle_t;
#define	BUS_SPACE_HANDLE_NULL	((bus_space_handle_t) 0)

/*
 *	All devices is located on a bus. When attached (or matched)
 *	the configuration routines send this structure as an argument
 *	(may be extended).
 */
typedef struct bus_attach_args {
	bus_t		ba_bus;		/* bus we're on */
	int32_t		ba_unit;	/* this is the unit */
} *bus_attach_args_t;

/*
 *	Prototypes:
 */
extern bus_t bus_create __P((
		device_t, 		/* owning device */
		bus_type, 		/* bus type */
		u_int32_t,		/* bus number */
		u_int32_t,		/* base */
		u_int32_t));		/* limit */
		
extern int bus_space_map __P((
		bus_t, 			/* bus */
		bus_space_tag_t *, 	/* tag out */
		bus_space_handle_t *,	/* handle out */
		u_int32_t,		/* offset */
		size_t));		/* length */
		
extern int bus_space_unmap __P((
		bus_space_tag_t,	/* tag */
		bus_space_handle_t));	/* handle */

extern int bus_space_map_pci_conf __P((
		bus_t,			/* bus */
		bus_space_tag_t *,	/* tag out */
		bus_space_handle_t *,	/* handle out */
		u_int32_t,		/* device */
		u_int32_t));		/* function */

extern int bus_space_map_pci __P((
		bus_t,			/* bus */
		bus_space_tag_t *,	/* tag out */
		bus_space_handle_t *,	/* handle out */
		u_int32_t,		/* offset */
		u_int32_t));		/* length */

extern void bus_space_write_1 __P((
		bus_space_tag_t,	/* tag */
		bus_space_handle_t,	/* handle */
		u_int32_t,		/* offset */
		u_int8_t));		/* value */

extern void bus_space_write_2 __P((
		bus_space_tag_t,	/* tag */
		bus_space_handle_t,	/* handle */
		u_int32_t,		/* offset */
		u_int16_t));		/* value */

extern void bus_space_write_4 __P((
		bus_space_tag_t,	/* tag */
		bus_space_handle_t,	/* handle */
		u_int32_t,		/* offset */
		u_int32_t));		/* value */

extern u_int8_t bus_space_read_1 __P((
		bus_space_tag_t,	/* tag */
		bus_space_handle_t,	/* handle */
		u_int32_t));		/* offset */

extern u_int16_t bus_space_read_2 __P((
		bus_space_tag_t,	/* tag */
		bus_space_handle_t,	/* handle */
		u_int32_t));		/* offset */

extern u_int32_t bus_space_read_4 __P((
		bus_space_tag_t,	/* tag */
		bus_space_handle_t,	/* handle */
		u_int32_t));		/* offset */

extern void bus_space_dump __P((
		bus_space_tag_t,	/* tag */
		bus_space_handle_t));	/* handle */

#endif /* BUS_H */
