#ifndef ROM_FS_H
#define	ROM_FS_H

/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: fs.h,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#include <sys/cdefs.h>
#include <sys/types.h>

#include <rom/device.h>

/*
 *	Filesystem actions:
 *
 *	A filesystem must implement the following
 *	actions: mount, unmount, openfile, readfile
 */
#define	FSA_MOUNT		0
#define	FSA_UNMOUNT		1
#define	FSA_OPENFILE		2
#define	FSA_CLOSEFILE		3
#define	FSA_READFILE		4
#define	FSA_WRITEFILE		5
#define	FSA_STAT		6
#define	FSA_OPENDIR		7
#define	FSA_CLOSEDIR		8
#define	FSA_READDIR		9

/*
 *	Filesystem action structure:
 *
 *	This structure is sent from the I/O system to
 *	the filesystem handler.
 */
typedef struct filesystem_action {
	device_unit_t	fsa_unit;	/* unit */
	char *		fsa_softc;	/* softc */
	unsigned int	fsa_action;	/* action */
	char *		fsa_buffer;	/* buffer */
	unsigned int	fsa_length;	/* length */
	unsigned int	fsa_actual;	/* actual length */
	unsigned int	fsa_error;	/* error */
	void *		fsa_file;	/* file */
} *filesystem_action_t;

/*
 *	The FS action handler: 
 */
typedef int (*fsa_handler_t)(filesystem_action_t);

/*
 *	Filesystem structure:
 */
typedef struct filesystem {
	char			*fs_name;	/* name of filesystem */
	device_unit_t		fs_device;	/* device we are attached to */
	fsa_handler_t		fs_fsah;	/* fsa handler */
	void			*fs_aux;	/* private data */
} *filesystem_t;
#define	FILESYSTEM_NULL	((filesystem_t) 0)

/*
 *	Filesystem attach structure:
 */
typedef struct filesystem_attach {
	char			*fa_name;	/* name */
	unsigned int		fa_length;	/* length of softc */
	fsa_handler_t		fa_fsah;	/* fsa handler */
} *filesystem_attach_t;
#define	FILESYSTEM_ATTACH_NULl	((filesystem_attach_t) 0)

/*
 *	Prototypes:
 */
extern filesystem_t filesystem_mount(
			char *,			/* device */
			int);			/* unit */

extern int filesystem_sendfsa(
			filesystem_t,		/* fs */
			filesystem_action_t);	/* fsa */

#endif /* ROM_FS_H */
