/* main.c

   Taken from OpenBIOS 0.0.1/boot/boot32/main.c which was in turn taken
   from /usr/src/linux/arch/i386/boot/compressed/misc.c

   Used for start32, 1/11/2000
   James Hendricks, Dale Webster */


/* misc.c
 * 
 * This is a collection of several routines from gzip-1.0.3 
 * adapted for Linux.
 *
 * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
 * puts by Nick Holloway 1993, better puts by Martin Mares 1995
 * High loaded stuff by Hans Lermen & Werner Almesberger, Feb. 1996
 */

#include "definitions.h"
#include "printk.h"
#include <params.h>

extern void intel_post(char value);

#define DEBUG
#define ERRCHK
#undef TRACEV
#define MALLOCDBG 

#ifdef DEBUG
#define DBG(x...) printk(KERN_DEBUG x)
#else
#define DBG(x...)
#endif

/*
 * gzip declarations
 */

#define OF(args)  args
#define STATIC static

#undef memset
#undef memcpy
#define memzero(s, n)     memset ((s), 0, (n))

typedef unsigned char uch;
typedef unsigned short ush;
typedef unsigned long ulg;

#define K64 (64*1024)
#define WSIZE 0x8000		/* Window size must be at least 32k, and a power of two */

uch *inbuf;			/* input buffer */
static uch window[WSIZE];	/* Sliding window buffer */

unsigned insize;		/* valid bytes in inbuf */
unsigned inptr;			/* index of next byte to be processed in inbuf */
unsigned outcnt;		/* bytes in output buffer */

/* gzip flag byte */
#define ASCII_FLAG   0x01	/* bit 0 set: file probably ASCII text */
#define CONTINUATION 0x02	/* bit 1 set: continuation of multi-part gzip file */
#define EXTRA_FIELD  0x04	/* bit 2 set: extra field present */
#define ORIG_NAME    0x08	/* bit 3 set: original file name present */
#define COMMENT      0x10	/* bit 4 set: file comment present */
#define ENCRYPTED    0x20	/* bit 5 set: file is encrypted */
#define RESERVED     0xC0	/* bit 6,7:   reserved */

extern int fill_inbuf(void);
extern void error(char[]);
extern void displayinit(void);

typedef void (*kernel) ();
kernel v;

void malloc_init(unsigned long start, unsigned long end);
void *malloc(int size);

static void flush_window(void);
static void free(void *where);
static void gzip_mark(void **);
static void gzip_release(void **);

static long bytes_out;
static uch *output_data;
static unsigned long output_ptr;

static long free_mem_ptr;	/* Unused memory */
static long free_mem_end_ptr;	/* 128k */

#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())

//static int verbose = 1;
/* Diagnostic functions */
#ifdef ERRCHK
#  define Assert(cond,msg) {if(!(cond)) error(msg);}
#else
#  define Assert(cond,msg)
#endif

#ifdef TRACEV
#  define Trace(x) printk(KERN_DDEBUG x)
#  define Tracev(x) {if (verbose) printk(KERN_DDEBUG  x);}
#  define Tracevv(x) {if (verbose>1) printk(KERN_DDEBUG  x);}
#  define Tracec(c,x) {if (verbose && (c)) printk(KERN_DDEBUG x);}
#  define Tracecv(c,x) {if (verbose>1 && (c)) printk(KERN_DDEBUG  x);}
#else
#  define Trace(x)
#  define Tracev(x)
#  define Tracevv(x)
#  define Tracec(c,x)
#  define Tracecv(c,x)
#endif

#include "inflate.c"

void malloc_init(unsigned long start, unsigned long end)
{
	free_mem_ptr = start;
	free_mem_end_ptr = end;
}

void *malloc(int size)
{
	void *p;

	if (size < 0)
		error("Error! malloc: Size < 0");
	if (free_mem_ptr <= 0)
		error("Error! malloc: Free_mem_ptr <= 0");

	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */

	p = (void *) free_mem_ptr;
	free_mem_ptr += size;

	if (free_mem_ptr >= free_mem_end_ptr)
		error("Error! malloc: Free_mem_ptr >= free_mem_end_ptr");

	MALLOCDBG("malloc 0x%08x\n", (unsigned int)p);

	return p;
}

static void free(void *where)
{
	/* Don't care */
}

static void gzip_mark(void **ptr)
{
	*ptr = (void *) free_mem_ptr;
	DBG("gzip_mark 0x%08x\n", (unsigned int)free_mem_ptr);
}

static void gzip_release(void **ptr)
{
	free_mem_ptr = (long) *ptr;
	DBG("gzip_release 0x%08x\n", (unsigned int)free_mem_ptr);
}

void *memset(void *s, int c, size_t n)
{
	int i;
	char *ss = (char *) s;

	for (i = 0; i < n; i++)
		ss[i] = c;

	return s;
}

void *memcpy(void *__dest, __const void *__src, size_t __n)
{
	int i;
	char *d = (char *) __dest, *s = (char *) __src;

	for (i = 0; i < __n; i++)
		d[i] = s[i];

	return __dest;
}

/* ===========================================================================
 * Write the output window window[0..outcnt-1] and update crc and bytes_out.
 * (Used for the decompressed data only.)
 */
static void flush_window()
{
	ulg c = crc;		/* temporary variable */
	unsigned n;
	uch *in, *out, ch;

	in = window;
	out = &output_data[output_ptr];
	DBG("flush 0x%08x count 0x%08x\n", (unsigned long) out, outcnt);

	for (n = 0; n < outcnt; n++) {
		ch = *out++ = *in++;
		c = crc_32_tab[((int) c ^ ch) & 0xff] ^ (c >> 8);
	}
	crc = c;
	bytes_out += (ulg) outcnt;
	output_ptr += (ulg) outcnt;
	outcnt = 0;
}

void setup_output_buffer()
{
	output_data = (char *) KERNEL_START;
	DBG("output data is 0x%08x\n", (unsigned long) output_data);
}

/* TODO: this must move to chip/intel */
/* we have to do more than we thought. I assumed Linux would do all the
 * interesting parts, and I was wrong. 
 */
struct ioapicreg {
	unsigned int reg;
	unsigned int value;
};

struct ioapicreg ioapicregvalues[] = {
	{0x10, 0x10000}, {0x11, 0x0},
	{0x12, 0x959}, {0x13, 0xff000000},
	{0x14, 0x951}, {0x15, 0xff000000},
	{0x16, 0x961}, {0x17, 0x0},
	{0x18, 0x969}, {0x19, 0xff000000},
	{0x1a, 0x971}, {0x1b, 0x0},
	{0x1c, 0x979}, {0x1d, 0x0},
	{0x1e, 0x981}, {0x1f, 0x0},
	{0x20, 0x989}, {0x21, 0xff000000},
	{0x22, 0x10000}, {0x23, 0x0},
	{0x24, 0x10000}, {0x25, 0x0},
	{0x26, 0x10000}, {0x27, 0x0},
	{0x28, 0x991}, {0x29, 0xff000000},
	{0x2a, 0x10000}, {0x2b, 0x0},
	{0x2c, 0x999}, {0x2d, 0x0},
	{0x2e, 0x9a1}, {0x2f, 0x0},
	{0x30, 0x10000}, {0x31, 0x0},
	{0x32, 0x10000}, {0x33, 0x0},
	{0x34, 0x10000}, {0x35, 0x0},
	{0x36, 0xa9a9}, {0x37, 0xff000000},
	{0x38, 0x10000}, {0x39, 0x0},
	{0x3a, 0xa9b1}, {0x3b, 0xff000000},
	{0x3c, 0x10000}, {0x3d, 0x0},
	{0x3e, 0x10000}, {0x3f, 0x0}
};

void setup_apic()
{
	int i;
	unsigned long l1;
	unsigned long nvram = 0xfec00000;
	volatile unsigned long *l;
	struct ioapicreg *a = ioapicregvalues;

	l = (unsigned long *) nvram;
	for (i = 0; i < sizeof(ioapicregvalues) / sizeof(ioapicregvalues[0]);
	     i++, a++) {
		*l = a->reg;
		l[4] = a->value;
		l1 = l[4];
		if ((i==0) && (l1 == 0xffffffff)) {
			DBG("IO APIC not responding.\n");
			return;
		}
		DBG("for IRQ, reg 0x%08x value 0x%08x\n", a->reg, l1);
	}
}

int
linuxbiosmain(unsigned long base, unsigned long totalram)
{
	unsigned char *empty_zero_page;
	extern int firstfill;

	/* common globals -- don't rely on init! */
	insize = 0;
	inptr = 0;
	firstfill = 1;
	outcnt = 0;
	bytes_out = 0;
	output_ptr = 0;

	printk("Welcome to start32, the open sourced starter.\n");
	printk("This space will eventually hold more diagnostic information.\n");
	printk("\n");
	printk("January 2000, James Hendricks, Dale Webster, and Ron Minnich.\n");
	printk("Version 0.1\n");
	printk("\n");

	setup_output_buffer();

	DBG("Making CRC\n");
	makecrc();
	intel_post(0xf1);

	DBG("Gunzipping boot code\n");
	if (gunzip() != 0) {
		printk("gunzip failed\n");
		intel_post(0xff);
		return 0;
	}
	intel_post(0xf8);

	/* parameter passing to linux. You have to get the pointer to the
	 * empty_zero_page, then fill it in. 
	 */
	/* should we do this in intel_main(). If we did we would have to save it
	 * maybe -- 0x90000 might get stomped. We do it here as the last step.
	 */
	/* yes I know we need prototypes -- we'll do it soon */
	empty_zero_page = get_empty_zero_page();
	init_params(empty_zero_page);
	intel_post(0xf9);

	/* the ram address should be the last mbyte, AFAIK. Subtract one for the
	 * low 1 MB. So subtract 2K total 
	 */
	set_memory_size(empty_zero_page, 0x3c00, totalram - 2048);
	intel_post(0xfa);
#ifdef CMD_LINE
	DBG("using custom command line - [%s]\n", CMD_LINE);
#else
#define CMD_LINE "root=/dev/hda1 single"
	DBG("using default command line - [%s]\n", CMD_LINE);
#endif

	set_command_line(empty_zero_page, CMD_LINE);
	set_root_rdonly(empty_zero_page);
	set_display(empty_zero_page, 25, 80);

	/* set up the IO-APIC for the clock interrupt. */
	/* this needs to move to intel_main.c at some point. */
	setup_apic();
	intel_post(0xfc);

	DBG("Jumping to boot code\n");
	intel_post(0xfe);

	/* there seems to be a bug in gas? it's generating wrong bit-patterns ...
	   v = (kernel)KERNEL_START;
	   v();
	*/
	/* move 0x90000 to into esi (This is the address of the linux parameter page)
	 * Linux then copies this page into it's ``empty_zero_page'' so it isn't
	 * stomped while things are being setup.
	 * Later the ``empty_zero_page'' is zeroed and used fulfill read demands
	 * on memory mappings of file holes and the like.
	 * As of 2.4.0-test4 the linux parameter page isn't hardwired to be
	 * at 0x90000 anymore.
	 */
	/* move 0 to ebx. This is for SMP support. Jump to kernel */
	__asm__ __volatile__("movl $0x90000, %%esi\n\t"
			     "movl $0, %%ebx\n\t"
			     "ljmp $0x10, %0\n\t"
			     :: "i" (0x100000));

	return 0;		/* It should not ever return */
}
