/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: boot_main.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: boot_main.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <sys/cdefs.h>
#include <sys/types.h>

#include <rom/device.h>
#include <rom/fs.h>

extern filesystem_t rootfs;

#include "default.h"

#if 1
#define	ENABLE_TABULATOR
#endif

/** limits: */
#define	SZBUFF		128
#define	NARGS		8

#ifndef VERSION
#define	VERSION		"0.0.1-alpha"
#endif

/*
 *	Prototypes:
 */
extern int configure __P((void));
extern char * malloc __P((int));

extern void stpc_conf_write __P((
			char	port,
			char	value));
extern char stpc_conf_read __P((
			char	port));

extern char *hal_cpu_type __P((
			void));

extern char *ide_softc[];

#define	TIMEOUT		-1

/*
 *	Commands that we support:
 */
static char *command_strs[] = {
	"boot",
	"devs",
	"help",
	0
};

#define	CMD_UNKNOWN	-1 
#define	CMD_BOOT	0
#define	CMD_DEVICES	1
#define	CMD_HELP	2

#ifdef ENABLE_TABULATOR
int tabulator(
	char	*buff,
	int	cnt,
	int	maxlen)
{
	char tbuff[SZBUFF], *s, *args[NARGS], *found = 0;
	int argc, i, tocnt; 

	/*
	 * First we have sanity check.
	 */
	if(cnt == 0) {
		putchar('\a');
		return cnt;
	}

	/*
	 * Parse the string.
	 */
	strncpy(tbuff, buff, sizeof tbuff);
	tbuff[cnt] = '\0';	/* mark end of string */

	for(argc = 0, s = tbuff; argc < NARGS; argc++) {
		s = (char *) strtok(s, " ");
		if(s == 0)
			break;
		args[argc] = s;
		s = 0;
	}
	args[argc] = 0;

#if 1
	/*
	 * Complete commands.
	 */
	if(argc == 1) {
		for(i = 0; ; i++) {
			s = command_strs[i];
			if(s == 0)
				break;
			if(strncmp(args[0], s, strlen(args[0])) == 0) {
				found = s;
			}
		}
		if(found == 0) {
			putchar('\a');
			return cnt;
		}
	}
#endif

	/*
	 * Copy the string. 
	 */
	if(found) {
		for(i = 0, tocnt = cnt; i < (strlen(found) - tocnt); i++) {
			*(buff + cnt++) = *(found + tocnt + i);
		}
		*(buff + cnt++) = ' ';
	}
	else {
		putchar('\a');
	}
	return cnt;
}
#endif

/*
 *	command_boot:
 */
int command_boot(
	char	**args,
	int	argc)
{
	if(argc > 1) 
		boot_kernel(args[1]);
}

/*
 *	This is the routine that prompts the user for what
 *	kernel the loader should load and run.
 */ 
int prompt(
	void)
{
	int i, cnt, first = 1;
	char buff[SZBUFF], *args[NARGS], *s, *p;

	printf("\r; [boot [[%s:]%s]] ", DEFDEVICE, DEFKERNEL);
	while(1) {
#ifdef ENABLE_TABULATOR
		i = readline(buff, 128, tabulator);
#else
		i = readline(buff, 128, 0);
#endif	
		if(first == 1 && (i == TIMEOUT || i == 0)) {
			/* load default kernel */
			printf("\n");
			boot_kernel(DEFCOMBI);
		}
		else if(i == TIMEOUT)
			continue;
		first = 0;		
		
		if(i > 0) {
			/*
			 * Parse the command line.
			 */
			for(cnt = 0, s = buff, p = buff; cnt < NARGS; cnt++) {
#if 0
				s = (char *) index(s, ' ');
				args[cnt] = p;
				if(s == 0) {
					cnt++;
					break;
				}
				
				*s++ = '\0';
				
				/** strip whitespaces */
				while(*s == ' ' && *s != '\0')
					s++;
				p = s;
#else
				s = (char *) strtok(s, " ");
				if(s == 0)
					break;
				args[cnt] = s;
				s = 0;
#endif
			}
			args[cnt] = 0;
			
			/*
			 * Check what command it was.
			 */
			if(cnt > 0) {
				switch(strarraycmp(args[0], command_strs)) {
					case CMD_UNKNOWN:
						printf("\nunknown command '%s'\n", buff);
						break;
				
					case CMD_BOOT:
						command_boot(args, cnt);
						break;
				
					case CMD_DEVICES:
						config_dump();
						break;

					case CMD_HELP:
						printf("\n--- availible commands:\n");
						for(i = 0 ;; i++) {
							if(command_strs[i] == 0)
								break;
							printf("    %s\n", command_strs[i]);
						}
						break;
				}
			}
		}
		
		/* 
		 * Print our prompt.
		 */
		printf("\r; ");
	}
	return 0;
}

/*
 *	The main routine. When we come here when the chipset has been 
 *	initialized and we can concentrate on geting the devices going 
 *	and booting the kernel.
 */
int
main()
{
	char *s, *d;

	/*
	 *	We initialize communication port 0, so that we
	 *	can use ``printf
	 */
	com_cons_init(0);

	/*
	 *	Seems to be a bug here. We must allocate 2*128 bytes before
	 *	we can start to configure the system.
	 */
#if 1
	s = malloc(128);
	d = malloc(128);
#endif


	/*
	 *	Autoconfigure the buses, bridges and devices.
	 */	
	(void) configure();

	/*
	 *	Print some information!
	 */
	printf("\n\n>>> Bootstrap version %s (%s %s)\n", VERSION, __DATE__, __TIME__);
	printf(">>> Copyright (c) 1999 by Net Insight AB. All Rights Reserved.\n");
	printf(">>> (%s)\n\n", hal_cpu_type());

	(void) prompt();

	for(;;);
	
	return 0;
}

