/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: boot_subr.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: boot_subr.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <rom/device.h>
#include <rom/fs.h>

#include <sys/endian.h>
#include <sys/aout.h>

#include "default.h"
#include "boot_subr.h"

/*
 * Number of arguments sent to the kernel.
 */
#define	NARGS		6

/*
 * Generated by GNU ld (from ldscript.ld) 
 */
extern char _KERNSTK[], _BOOTINFO[];

/*
 *
 */
struct btinfo_table {
	unsigned int	entries;
	u_int32_t	addr[1];
} btinfo_table;

/*
 *
 */
#define	BTINFO_ADD(what, typ, size)				\
	((struct btinfo_common *)what)->type = typ;		\
	((struct btinfo_common *)what)->len  = (size);		\
	btinfo_table.addr[btinfo_table.entries++] = what

/*
 * boot_kernel:
 */
void boot_kernel(
	char	*fname)
{
	extern filesystem_t rootfs;
	char dev[16], file[32], *s, *p = file;
	int len, unit = 0, ret, fd;
	char buff[32];
	u_int32_t marks[MARK_MAX];
	char	*argv[NARGS];

	/*
	 *	Headers:
	 */
	union {
		struct	exec	aout;
	} hdr;

	/*
	 *
	 */
	struct btinfo_console *cons;
	
	/*
	 * XXX
	 */
	btinfo_table.entries = 0;

	/*
	 *
	 */
	cons = malloc(sizeof *cons);
	strcpy(cons->devname, "com0");
	cons->addr = 0x3f8;
	cons->speed = 9600;
	BTINFO_ADD(cons, BTINFO_CONSOLE, sizeof *cons);
	
	/*
	 * Default names.
	 */
	strcpy(dev, DEFDEVICE);
	strcpy(file, DEFKERNEL);

	/*
	 *	Parse the device.
	 */
	if((s = (char *) index(fname, ':')) != 0) {
		strncpy(dev, fname, (len = ((int) index(fname, ':') - (int) fname)));
		dev[len] = '\0';
	
		p = fname + len + 1;
	}
	else {
		p = fname;
	}

	strncpy(file, p, sizeof file);		
	file[strlen(p)] = '\0';
		
	/*
	 *	Parse unit.
	 */
	unit = dev[3] - 'a';
	dev[3] = '\0';

	/*
	 *	Print some debug information.
	 */
#if 0
	printf("\nboot_kernel: dev = '%s' unit = %d, '%s'\n", dev, unit, p);
#endif

	/*
	 *
	 */
	printf("\nTries to load kernel file '%s': ", fname);
		
	/*
	 *	Try to mount the filesystem.
	 */
	rootfs = filesystem_mount(dev, unit);
	if(rootfs == 0) {
		printf("Couldn't mount device %s%c\n",
			dev, 'a' + unit);
		return;
	}

	fd = open(file, 0);
	if(fd < 0) {
		printf("Can't open file\n");
		return;
	}
	
	len = read(fd, &hdr, sizeof hdr);
#if 0
	printf("len = %d\n", len);
#endif
	
	marks[MARK_START] = 0x0;

	if(N_GETMAGIC(hdr.aout) == ZMAGIC  &&
	    N_GETMID(hdr.aout) == MID_I386) {
		ret = aout_exec(fd, &hdr.aout, marks, LOAD_ALL);
	}       
	else {
		printf("File is invalid\n");
		return;
	}                                  
		
	/*
	 *	Setup arguments:
	 */
	argv[0]		= 0;		/* boot howto */
	argv[1]		= 0;		/* root device */
	argv[2]		= 0;		/* boot info */
	argv[3]		= 0;		/* esym */
	argv[4]		= 1024 * 15;	/* ext mem */
	argv[5]		= 634;		/* base mem */

#if 0
	printf("\nmark1: %#8.8x\n", marks[0]);
	printf("mark2: %#8.8x\n", marks[1]);
	printf("mark3: %#8.8x\n", marks[2]);
	printf("mark4: %#8.8x\n", marks[3]);
	printf("mark5: %#8.8x\n", marks[4]);	
#endif
	
#if 1
	if(ret == 0) {
		printf(": jumping to %#x\n\n", 
			marks[MARK_ENTRY]);
#if 0
		hexdump("kernel", marks[MARK_ENTRY], 64);
#else
		startkernel(marks[MARK_ENTRY], 6, argv, _KERNSTK);
#endif
	}
#endif	
	printf(": Invalid kernel\n");
}

static int aout_exec(
	int		fd,
	struct exec	*x,
	u_int32_t	*marks,
	int		flags)
{
	u_long entry = x->a_entry;
	u_int32_t aoutp = 0;
	u_int32_t minp, maxp;
	int cc;
	u_int32_t offset = marks[MARK_START];
	u_long magic = N_GETMAGIC(*x);
	int sub;

	/* In OMAGIC, exec header isn't part of text segment */
	if (magic == OMAGIC)
		sub = 0;
	else
		sub = sizeof(*x);
	
	minp = maxp = ALIGNENTRY(entry);
		
	if (seek(fd, sizeof(*x)) == -1)  {
		printf("lseek text");
		return 1;
	}

	/*
	 * Leave a copy of the exec header before the text.
	 * The kernel may use this to verify that the
	 * symbols were loaded by this boot program.
	 */
	if (flags & LOAD_HDR)
		BCOPY(x, maxp, sizeof(*x));
	if (flags & (LOAD_HDR|COUNT_HDR))
		maxp += sizeof(*x);

	/*
	 * Read in the text segment.
	 */
	if (flags & LOAD_TEXT) {
		PROGRESS(("%ld", x->a_text));

		if (READ(fd, maxp, x->a_text - sub) != x->a_text - sub) {
			WARN(("read text"));
			return 1;
		}
	} else {
		if (seek(fd, x->a_text - sub) == -1) {
			WARN(("seek text"));
			return 1;
		}
	}
	if (flags & (LOAD_TEXT|COUNT_TEXT))
		maxp += x->a_text - sub;

	/*
	 * Read in the data segment.
	 */
	if (flags & LOAD_DATA) {
		PROGRESS(("+%ld", x->a_data));

		if (READ(fd, maxp, x->a_data) != x->a_data) {
			WARN(("read data"));
			return 1;
		}
	}
	else {
		if (seek(fd, x->a_data) == -1) {
			WARN(("seek data"));
			return 1;
		}
	}
	if (flags & (LOAD_DATA|COUNT_DATA))
		maxp += x->a_data;

	/*
	 * Zero out the BSS section.
	 * (Kernel doesn't care, but do it anyway.)
	 */
#if 0
	if (flags & LOAD_BSS) {
		PROGRESS(("+%ld", x->a_bss));

		BZERO(maxp, x->a_bss);
	}
#endif

	if (flags & (LOAD_BSS|COUNT_BSS))
		maxp += x->a_bss;

	/*
	 * Read in the symbol table and strings.
	 * (Always set the symtab size word.)
	 */
	if (flags & LOAD_SYM)
		BCOPY(&x->a_syms, maxp, sizeof(x->a_syms));

	if (flags & (LOAD_SYM|COUNT_SYM)) {
		maxp += sizeof(x->a_syms);
		aoutp = maxp;
	}

	if (x->a_syms > 0) {
		/* Symbol table and string table length word. */

		if (flags & LOAD_SYM) {
			PROGRESS(("+[%ld", x->a_syms));

			if (READ(fd, maxp, x->a_syms) != x->a_syms) {
				WARN(("read symbols"));
				return 1;
			}
		} else  {
			if (seek(fd, x->a_syms) == -1) {
				WARN(("seek symbols"));
				return 1;
			}
		}
		if (flags & (LOAD_SYM|COUNT_SYM))
			maxp += x->a_syms;

		if (read(fd, &cc, sizeof(cc)) != sizeof(cc)) {
			WARN(("read string table"));
			return 1;
		}

		if (flags & LOAD_SYM) {
			BCOPY(&cc, maxp, sizeof(cc));

			/* String table. Length word includes itself. */

			PROGRESS(("+%d]", cc));
		}
		if (flags & (LOAD_SYM|COUNT_SYM))
			maxp += sizeof(cc);

		cc -= sizeof(int);
		if (cc <= 0) {
			WARN(("symbol table too short"));
			return 1;
		}

		if (flags & LOAD_SYM) {
			if (READ(fd, maxp, cc) != cc) {
				WARN(("read strings"));
				return 1;
			}
		} else {
			if (seek(fd, cc) == -1) {
				WARN(("seek strings"));
				return 1;
			}
		}
		if (flags & (LOAD_SYM|COUNT_SYM))
			maxp += cc;
	}

	marks[MARK_START] = LOADADDR(minp);
	marks[MARK_ENTRY] = LOADADDR(entry);
	marks[MARK_NSYM] = x->a_syms;
	marks[MARK_SYM] = LOADADDR(aoutp);
	marks[MARK_END] = LOADADDR(maxp);
	return 0;
}


/*
 *
 */
void progress()
{
	static int cnt = -1;
	char *chars = "|/-\\|/-\\";

	cnt = (++cnt & 7);
	putchar(chars[cnt]);
	putchar('\b');
}
