/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: bus_pci.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: bus_pci.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <rom/device.h>
#include <rom/bus/bus.h>
#include <rom/bus/pci.h>

/*
 *	PCI bus device private data:
 */
typedef struct pcibus_softc {
	bus_t			sc_bus;		/* our bus */
	bus_t			sc_parentbus;	/* parent bus */
} *pcibus_softc_t;

/*
 *	Prototypes:
 */
extern unsigned long	pci_maketag __P((
		unsigned char	bus,
		unsigned char	device,
		unsigned char	function));

/*
 *	Check if we are a PCI bus.
 */
int pcibusprobe(
	device_t	parent,
	cfdata_t	cf,
	void		*aux)
{
	struct pci_attach_args *pa = (struct pci_attach_args *) aux;	
	unsigned long tmp, tag, cnt = 0, err;

	/*
	 * Check so that we really have a PCI bus!
	 */
	if(pa->pa_bargs.ba_unit == 0) 
		return CF_MATCH_EXACT;

	/*
	 * We can not just only act as a PCI bus, but also
	 * as a PCI-to-PCI bridge.
	 */
	if(pa->pa_class == PCI_CLASS_BRIDGE && pa->pa_subclass == PCI_SUBCLASS_BRIDGE_PCI) {
		return CF_MATCH_EXACT;
	}
	return CF_MATCH_NONE;
}

/*
 *	Attach routine.
 */
int32_t pcibusattach(
	device_t	parent,
	device_t	self,
	void		*aux)
{
	struct pci_attach_args *pa = (struct pci_attach_args *) aux;
	pcibus_softc_t sc = (pcibus_softc_t) self->dv_softc;
	struct pci_attach_args npa;
	int ret, flags;
	
	/*
	 * Base and limit of bridge.
	 */
	u_int32_t base = PCI_ADDRESS_BASE, limit = PCI_ADDRESS_LIMIT;
	
	/*
	 * Device information.
	 */
	u_int32_t bhlcr, id, class;

	/*
	 * This is not zero if we are a bridge.
	 */
	int bridge = 0;
	
	/*
	 * Vendor and device ID.
	 */
	int vendor;
	
	/*
	 * We loop through all the slots to see if there exists
	 * any devices on this bus.
	 */
	int slot, nslots = 32;
	
	/*
	 * We must also loop through all functions if the
	 * device is a multifunction device.
	 */
	int function, nfunctions = 8;

	/*
	 * PCI tag.
	 */
	pci_tag_t tag;

	/*
	 * Print some information about the device.
	 */
#if 1
	device_print(self, &pa->pa_bargs);
	printf(": attaching\n");
#endif

	/*
	 * Copy information from the argument structure.
	 */
	sc->sc_parentbus = pa->pa_bargs.ba_bus;
	if(sc->sc_parentbus->bus_type == BUS_TYPE_PCI) {
		base	= sc->sc_parentbus->bus_addr_base;
		limit	= sc->sc_parentbus->bus_addr_limit;
	}


	/*
	 * XXX bus creation.
	 */
	sc->sc_bus = (bus_t) bus_create(self,
				BUS_TYPE_PCI,
				pa->pa_bargs.ba_unit,	/* fixed unit for now I guess */
				base,
				limit);	

	/*
	 * Loop through all the slots.
	 */
	for(slot = 0; slot < nslots; slot++) {
		/*
		 * XXX get information
		 */
		tag = pci_maketag(sc->sc_bus->bus_number,	/* bus */
				slot,				/* device */
				0);				/* function */

		/*
		 * Check for valid vendor ID. If not valid we continue
		 * to the next slot.
		 */
		id = pci_conf_read_4(tag, PCI_CONF_REGISTER_ID);
		if(PCI_VENDOR(id) == 0xffff || PCI_VENDOR(id) == 0) 
			continue;

		/*
		 * Check if we this is a multifunction device.
		 */
		nfunctions = 1;
		bhlcr = pci_conf_read_4(tag, PCI_CONF_REGISTER_BHLC);
		if(PCI_HDRTYPE_MULTIFN(bhlcr))
			nfunctions = 8;

	
		/*
		 * Loop through all functions.
		 */
		for(function = 0; function < nfunctions; function++) {
			tag = pci_maketag(sc->sc_bus->bus_number,
					slot,
					function);

			/*
			 * Initialize argument.
			 */
			npa.pa_bargs.ba_bus	= sc->sc_bus;
			npa.pa_bargs.ba_unit	= -1;

			/*
			 * Read vendor and product information from the
			 * function / device. Continue to next function if
			 * device is invalid.
			 */
			id = pci_conf_read_4(tag, PCI_CONF_REGISTER_ID);
			
			npa.pa_vendorid		= PCI_VENDOR(id);
			npa.pa_productid	= PCI_PRODUCT(id);
			if(PCI_VENDOR(id) == 0xffff || PCI_VENDOR(id) == 0) 
				continue;
			
			class = pci_conf_read_4(tag, PCI_CONF_REGISTER_CLASS);
			npa.pa_class		= PCI_CLASS(class);
			npa.pa_subclass		= PCI_SUBCLASS(class);
			npa.pa_interface	= PCI_INTERFACE(class);
			npa.pa_revision		= PCI_REVISION(class);

			npa.pa_tag		= tag;

#if 0
			printf("bus%d/%d/%d: vendor = %#x, product = %#x, class = %#x, subclass = %#x\n",
					sc->sc_bus->bus_number, slot, function,
					npa.pa_vendorid, npa.pa_productid, npa.pa_class, 
					npa.pa_subclass);
			printf("bus%d/%d/%d: interface = %d, revision = %d\n", 
					sc->sc_bus->bus_number, slot, function,
					npa.pa_interface, npa.pa_interface);
#endif

			/*
			 * Configure all devices on the PCI bus. We don't
			 * need a submatch routine since we're have done
			 * all needed configuration already.
			 */
			ret = config_search(self,	/* isabus */
					0,		/* routine */
					(void *) &npa);	/* aux */
		}
	}
	return 0;
}

/*
 *	Configuration data for the ISA bus.
 */
struct cfattach ca_pcibus = {
	sizeof(struct pcibus_softc), 	/* sizeof device private data */
	pcibusprobe,			/* match rotuine */
	pcibusattach,			/* attach routine */
	0,				/* detach routine */
	"pcibus"			/* name of "device" */
};
