/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: subr_autoconf.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: subr_autoconf.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <rom/bus/bus.h>
#include <rom/device.h>

/** from device_conf.c: */
extern struct cfdata device_cfdev[];

/*
 *	All devices:
 */
queue_head_t alldevs;

#define	STAR	-1

/*
 *
 */
int config_attach(
	device_t	parent,
	cfdata_t	cf,
	void		*aux)
{
	struct bus_attach_args *ba = (struct bus_attach_args *) aux;
	device_t self;
	int ret;
	
	/*
	 *	Allocate memory for the device structure.
	 */
	if((self = (device_t) malloc(sizeof *self)) == 0) 
		return 1;

	/*
	 *	Fill the new structure with information.
	 */
	self->dv_cfdata	= cf;
	self->dv_unit	= ba->ba_unit;
	self->dv_parent	= parent;
	self->dv_flags	= 0;
	
	/*
	 *	Allocate memory for the devices private data.
	 */
	self->dv_softc	= (char *) malloc(cf->cf_attach->ca_size);

	sprintf(self->dv_name, "%s%d", cf->cf_attach->ca_name, 
		ba->ba_unit);
		
	/*
	 *	XXX
	 */
	cf->cf_driver->cd_nunits++;
	
	/*
	 *	Now we call the attach routine.
	 */
	if(cf->cf_unit >= 0) 
		cf->cf_flags	= CF_FLAG_FOUND;
	(void) (*cf->cf_attach->ca_attach)(parent, self, aux);
	enqueue(&alldevs, &self->dv_link);
}

/*
 *
 */
int config_match(
	device_t	parent,
	cfdata_t	cf,
	int		(*match)(device_t, struct cfdata *, void *),
	void		*aux)
{
	struct bus_attach_args *ba = (struct bus_attach_args *) aux;

	ba->ba_unit		= cf->cf_unit;
	if(ba->ba_unit < 0)
		ba->ba_unit	= cf->cf_driver->cd_nunits;

	if(match) 
		return (*match)(parent, cf, aux);
	else
		return (*cf->cf_attach->ca_match)(parent, cf, aux);
}

/*
 *
 */
int config_search(
	device_t	parent,
	int		(*match)(device_t, struct cfdata *, void *),
	void		*aux)
{
	device_t device = parent;
	int pri = CF_MATCH_NONE, ret;
	struct cfdata *cf, *found = 0;

	for(cf = device_cfdev; cf->cf_attach; cf++) {
		if(cf->cf_parent && cf->cf_parent->cf_driver == device->dv_cfdata->cf_driver && cf->cf_flags == CF_FLAG_NOTFOUND) {
#if 0
			if(cf->cf_unit != STAR) {
				(void) config_attach(parent, cf, aux);
				continue;
			}
#endif
			
			ret = config_match(parent, cf, match, aux);
#if 0
			if(ret == CF_MATCH_EXACT) {
				(void) config_attach(parent, found, aux);
				found = 0;
				ret = CF_MATCH_NONE;			
			}
#endif
			if(ret > pri) {
				found = cf;	/* we found a device */
				pri = ret;	/* set current match rate */
			}
		}
	}

	if(found)
		(void) config_attach(parent, found, aux);
	else return -1;
	return 0;
}	

/*
 *
 */
int config_bus_match(
	device_t	parent, 
	struct cfdata	*cfdata,  
	void 		*aux)
{
	printf("%s%d\n", cfdata->cf_attach->ca_name,
		cfdata->cf_unit);

	/*
	 * We do nothing really!
	 */ 
	return (*cfdata->cf_attach->ca_match)(parent, cfdata, aux);
}

/*
 *
 */
void config_dump()
{
	device_t dev;

	printf("--- devices:\n");
	dev = (device_t) queue_first(&alldevs);
	while(!queue_end(&alldevs, &dev->dv_link)) {
		printf(" - %s\n", dev->dv_name);
	
		dev = (device_t) queue_next(&dev->dv_link);
	}
}

/*
 *
 *
 *
 */
void configure()
{
	struct bus_attach_args ba;
	struct cfdata *cf;

	/*
	 *	Initialize the queue of devices.
	 */
	queue_init(&alldevs);

	/*
	 *	Print some information.
	 */
	printf("\nautoconfigure...\n");

	/*
	 *	First of all we initialize mainbus0. 
	 */	
	ba.ba_bus	= BUS_NULL;
	ba.ba_unit	= 0;
	
	config_attach(DEVICE_NULL,	/* parent */
		&device_cfdev[0],	/* mainbus0 */
		&ba);
}
