/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: subr_io.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: subr_io.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <sys/cdefs.h>
#include <sys/stdarg.h>

/*
 *	Prototypes:
 */
extern void com_cons_putchar __P((
	int	port,
	char	ch));
extern int com_cons_trygetchar __P((
	int	port));

/*
 *
 */
void putchar(
	int	ch)
{
	com_cons_putchar(0, ch);
}

int printf(
	const char	*fmt,
	...)
{
	va_list args;
	int err;
	
	va_start(args, fmt);
	err = _doprnt(fmt, args, 0, (void (*)()) com_cons_putchar, (char *) 0);
	va_end(args);
	
	return err;
}

/*
 *	A readline routine. Not that complicated. 
 */
int readline(
	char		*buff,
	int		maxlen,
	int		(*tab)(char *, int, int))
{
	int cnt = 0, ch, gotone = 0, tocnt = 0;
	char tmpbuff[128];

	/*
	 * A simple forever-loop that breaks when we get
	 * a carrier return or a linefeed.
	 */
	while(1) {
		/*
		 * Check if we got a character.
		 */
		while((ch = com_cons_trygetchar(0)) < 0)
			if(gotone == 0) {
				if(tocnt++ > 0x3000)
					return -1;
			}
		gotone = 1;
		
		switch(ch) {
			/*
			 * Carrier return or linefeed.
			 */
			case '\r': case '\n':
				*(buff + cnt) = '\0';
				return cnt;
			
			/*
			 * Backstep.
			 */
			case '\b':
				/** check that we don't go 
				  * back to long */
				if(cnt > 0) {
					printf("\b ");
					cnt--;
				}
				/** if we do, alarm the user */
				else goto alarm;
				
				/** print the character. */
				goto print;
		
		 	/*
		 	 * Tab.
		 	 */ 
		 	case '\t':
		 		if(tab != 0) {
		 			*(buff + cnt) = '\0';
		 			tocnt = (*tab)(buff, cnt, maxlen);
		 			strncpy(tmpbuff, buff+cnt, (tocnt - cnt));
		 			tmpbuff[(tocnt-cnt)] = '\0';
		 			printf("%s", tmpbuff);
		 			cnt = tocnt;
		 		}
		 		break;
		
			/*
			 * A normal character.
			 */
			default:
				if(cnt == maxlen)
					goto alarm;
				*(buff + cnt++) = ch;
				goto print;
				
			/*
			 * Ooh! Better tell the damn user not to
			 * do this again!
			 */
			alarm:
				ch = '\a';
				/*FALLTHROUGH*/

			/*
			 * Print the character!
			 */
			print:
				putchar(ch);
				break;
		}
	}
}

/*
 *	panic:
 */
void panic(
	char *str)
{
	int cnt;
	
	printf("panic: %s\n", str);
	while(1);
}

