/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: subr_pci.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: subr_pci.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <sys/cdefs.h>
#include <sys/types.h>

#include <rom/bus/pci.h>

/*
 *	Prototypes:
 */
extern void outl __P((u_int16_t, u_int32_t));
extern u_int32_t inl __P((u_int16_t));

/*
 *	pci_maketag:
 *
 *	Creates a "tag" that can be sent to the PCI control index
 *	port.
 */
pci_tag_t pci_maketag(
	u_int32_t	bus,		/* bus number */
	u_int32_t	device,		/* device number */
	u_int32_t	function)	/* function number */
{
	return (pci_tag_t) (0x80000000 | bus << 16 | device << 11 | function << 8);
}

u_int32_t pci_conf_read_4(
	pci_tag_t	tag,		/* pci tag */
	u_int8_t	offset)		/* offset */
{
	outl(PCI_CONF_PORT_INDEX, tag | (offset & ~3));
	return inl(PCI_CONF_PORT_DATA);
}

u_int16_t pci_conf_read_2(
	pci_tag_t	tag,		/* pci tag */
	u_int8_t	offset)		/* offset */
{
	outl(PCI_CONF_PORT_INDEX, tag | (offset & ~3));
	return inw(PCI_CONF_PORT_DATA + (offset & 2));
}

u_int8_t pci_conf_read_1(
	pci_tag_t	tag,		/* pci tag */
	u_int8_t	offset)		/* offset */
{
	outl(PCI_CONF_PORT_INDEX, tag | (offset & ~3));
	return inb(PCI_CONF_PORT_DATA + (offset & 3));
}

void pci_conf_write_4(
	pci_tag_t	tag,		/* pci tag */
	u_int8_t	offset,		/* offset */
	u_int32_t	value)		/* value */
{
	outl(PCI_CONF_PORT_INDEX, tag | (offset & ~3));
	outl(PCI_CONF_PORT_DATA, value);
}

void pci_conf_write_2(
	pci_tag_t	tag,		/* pci tag */
	u_int8_t	offset,		/* offset */
	u_int16_t	value)		/* value */
{
	outl(PCI_CONF_PORT_INDEX, tag | (offset & ~3));
	outw(PCI_CONF_PORT_DATA + (offset & 2), value);
}

void pci_conf_write_1(
	pci_tag_t	tag,		/* pci tag */
	u_int8_t	offset,		/* offset */
	u_int16_t	value)		/* value */
{
	outl(PCI_CONF_PORT_INDEX, tag | (offset));
	outb(PCI_CONF_PORT_DATA + (offset & 3), value );
}
