/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: subr_serial.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: subr_serial.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

static int ser_io_base[4];

int com_cons_trygetchar(
	int port)
{
	int ch = -1;

	if (inb(ser_io_base[port] + 5) & 0x01)
		ch = inb(ser_io_base[port] + 0);
	return ch;
}

int com_cons_getchar(
	int port)
{
	int ch;

	/* Wait for a character to arrive.  */
	for (;;) {
		if (inb(ser_io_base[port] + 5) & 0x01) {
			/* Grab it.  */
			ch = inb(ser_io_base[port] + 0);
			break;
		}
	}
	return ch;
}

void com_cons_putchar(
	int port, 
	int ch)
{
	if (ch == '\n')
		com_cons_putchar(port,'\r');

	/* Wait for the transmit buffer to become available.  */
	while (!(inb(ser_io_base[port] + 5) & 0x20));

	outb(ser_io_base[port] + 0, ch);
}

void com_cons_init(
	int port)
{
	unsigned char dfr;
	unsigned divisor;

	switch (port)
	{
		case 0:	ser_io_base[0] = 0x3f8; break;
		case 1:	ser_io_base[1] = 0x2f8; break;
		case 2:	ser_io_base[2] = 0x3e8; break;
		case 3:	ser_io_base[3] = 0x2e8; break;
	}

#if 0
	/* Determine what to plug in the data format register.  */
	if (com_params->c_cflag & PARENB)
		if (com_params->c_cflag & PARODD)
			dfr = 0x08;
		else
			dfr = 0x18;
	else
#endif
		dfr = 0x03;
#if 0
	if (com_params->c_cflag & CSTOPB)
		dfr |= 0x04;
	switch (com_params->c_cflag & 0x00000300)
	{
		case CS5: dfr |= 0x00; break;
		case CS6: dfr |= 0x01; break;
		case CS7: dfr |= 0x02; break;
		case CS8: dfr |= 0x03; break;
	}
#endif

#if 0
	/* Convert the baud rate into a divisor latch value.  */
	divisor = 115200 / com_params->c_ospeed;
#else
	divisor = 115200 / 9600;
#endif

	/* Initialize the serial port.  */
	outb(ser_io_base[port] + 3, 0x80 | dfr);	/* DLAB = 1 */
	outb(ser_io_base[port] + 0, divisor & 0xff);
	outb(ser_io_base[port] + 1, divisor >> 8);
	outb(ser_io_base[port] + 3, 0x03 | dfr);	/* DLAB = 0, frame = 8N1 */
	outb(ser_io_base[port] + 1, 0x00);	/* no interrupts enabled */
	outb(ser_io_base[port] + 4, 0x0b);	/* OUT2, RTS, and DTR enabled */

	/* make sure the FIFO is on */
	outb(ser_io_base[port] + 2, 0x41); /* 4 byte trigger (0x40); on (0x01) */

	/* Clear all serial interrupts.  */
	inb(ser_io_base[port] + 6);	/* ID 0: read RS-232 status register */
	inb(ser_io_base[port] + 2);	/* ID 1: read interrupt identification reg */
	inb(ser_io_base[port] + 0);	/* ID 2: read receive buffer register */
	inb(ser_io_base[port] + 5);	/* ID 3: read serialization status reg */
}

/*
 * This delays until the transmit queue has emptied on the serial port.
 * XXX: It could potentially delay forever. . .
 */
void com_cons_flush(
	int port)
{
	/*
	 * Serialization Status Register:
	 * bit 6 is 1 if no byte in hold register or shift register.
	 */
	while (!(inb(ser_io_base[port] + 5) & 0x40))
		;
}

