/*
 * Bootstrap code for the STPC Consumer
 * Copyright (c) 1999 by Net Insight AB. All Rights Reserved.
 *
 * $Id: subr_strings.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $
 *
 */

#ifndef lint
static char rcsid[] = "$Id: subr_strings.c,v 1.2 2000/03/05 21:52:13 jgarzik Exp $"; 
#endif

#include <sys/stdarg.h>

#ifndef NULL
#define	NULL	0
#endif

#define SPRINTF_UNLIMITED -1
struct sprintf_state {
      char *buf;
      unsigned int len;
      unsigned int max;
};

int strcmp(
	char	*s1, 
	char	*s2)
{
	register unsigned int a, b;

	while((a = *s1++), (b = *s2++), a && b) {
		if (a != b)
			return (a-b);
	}
	return a-b;
}

int strarraycmp(
	char	*s1,
	char	**array)
{
	int cnt = 0;

	while(*array) {
		if(strcmp(s1, *array) == 0)
			return cnt;
		array++; cnt++;
	}
	return -1;
}

static void savechar(
	char	*arg, 
	int	c)
{
	struct sprintf_state *state = (struct sprintf_state *)arg;
	
	if (state->max != SPRINTF_UNLIMITED)
	{
		if (state->len == state->max)
			return;
	}

	state->len++;
	*state->buf = c;
	state->buf++;
}

int vsprintf(
	char		*s, 
	const char	*fmt, 
	va_list		args)
{
	struct sprintf_state state;
	state.max = SPRINTF_UNLIMITED;
	state.len = 0;
	state.buf = s;

	_doprnt(fmt, args, 0, (void (*)()) savechar, (char *) &state);
	*(state.buf) = '\0';

	return state.len;
}

int vsnprintf(
	char		*s, 
	int		size, 
	const char	*fmt, 
	va_list		args)
{
	struct sprintf_state state;
	state.max = size;
	state.len = 0;
	state.buf = s;

	_doprnt(fmt, args, 0, (void (*)()) savechar, (char *) &state);
	*(state.buf) = '\0';

	return state.len;
}

int sprintf(
	char		*s, 
	const char	*fmt, 
	...)
{
	va_list	args;
	int err;

	va_start(args, fmt);
	err = vsprintf(s, fmt, args);
	va_end(args);

	return err;
}

int snprintf(
	char		*s, 
	int		size, 
	const char	*fmt, 
	...)
{
	va_list	args;
	int err;

	va_start(args, fmt);
	err = vsnprintf(s, size, fmt, args);
	va_end(args);

	return err;
}

char *strtok(s, delim)
	register char *s;
	register const char *delim;
{
	register char *spanp;
	register int c, sc;
	char *tok;
	static char *last;


	if (s == NULL && (s = last) == NULL)
		return (NULL);

	/*
	 * Skip (span) leading delimiters (s += strspn(s, delim), sort of).
	 */
cont:
	c = *s++;
	for (spanp = (char *)delim; (sc = *spanp++) != 0;) {
		if (c == sc)
			goto cont;
	}

	if (c == 0) {		/* no non-delimiter characters */
		last = NULL;
		return (NULL);
	}
	tok = s - 1;

	/*
	 * Scan token (scan for delimiters: s += strcspn(s, delim), sort of).
	 * Note that delim must have one NUL; we stop if we see that, too.
	 */
	for (;;) {
		c = *s++;
		spanp = (char *)delim;
		do {
			if ((sc = *spanp++) == c) {
				if (c == 0)
					s = NULL;
				else
					s[-1] = 0;
				last = s;
				return (tok);
			}
		} while (sc != 0);
	}
	/* NOTREACHED */
}

char *strncpy(
	char		*to, 
	const char	*from, 
	int		count)
{
	register char *ret = to;

	while (count > 0) {
		count--;
		if ((*to++ = *from++) == '\0')
			break;
	}

	while (count > 0) {
		count--;
		*to++ = '\0';
	}
	return ret;
}

int strncmp(
	const char	*s1, 
	const char	*s2, 
	int		n)
{
	while (1) {
		if (n <= 0)
			return 0;
		if (*s1 != *s2)
			return *s1 - *s2;
		if (*s1 == 0)
			return 0;

		s1++; s2++; n--;
	}
}

char *index(
	char	*s,
	char	c)
{
	register const unsigned char *p = s;

	while(*p != 0) {
		if (*p++ == c)
			return ((void *)(p - 1));
	}
	return 0;
}

void hexdump(
	char		*name,
	unsigned char	*buff,
	int		len)
{
	char line[128];
	int offset = 0, cnt = 0, first = 1;
	
	printf("hex dump of %s:", name);
	while(offset < len) {
		if((offset % 32) == 0) {
			line[cnt] = 0;
			if(first == 0) {
				printf("   %s", line);
			}
			printf("\n%8.8x: ", offset);
			first = cnt = 0;
		}
		
		line[cnt++] = (*buff >= 0x20 && *buff < 0x80) ? *buff : '.';
		
		printf("%2.2x ", *buff++ & 0xff);
		offset++;
	}
	line[cnt] = 0;
	printf("   %s\n", line);
}
