#include <cpu/p5/io.h>
#include <printk.h>
#include <types.h>
/* this is an architecture-dependent function now. There are too many
 * motherboard differences
 */
/* ===========================================================================
 * Fill the input buffer. This is called only when the buffer is empty
 * and at least one byte is really needed.
 */

#ifdef USE_DOC_MIL
#define DOC_KERNEL_START 65536
#endif

#ifndef ZKERNEL_START
#define ZKERNEL_START 0xfff80000
#endif

#ifndef ZKERNEL_MASK
#define ZKERNEL_MASK 0x0000ffff
#endif

extern void intel_post(char value);

static unsigned char *nvram;
static int block_count;
int firstfill;

#ifdef USE_DOC_MIL
void memcpy_from_doc_mil(void *dest, const void *src, size_t n);
unsigned char *doc_mil = (unsigned char *) 0xffffe000;
#endif

#ifdef INBUF_COPY
static unsigned char *ram;
extern char *malloc();
#endif

#define K64 (64 * 1024)

#ifndef USE_DOC_MIL
int fill_inbuf()
{
	extern unsigned char *inbuf;
	extern unsigned int insize;
	extern unsigned int inptr;

	if (firstfill) {
		block_count = 0;
		firstfill = 0;
#ifdef INBUF_COPY
		ram = malloc(K64);
#endif
	}

	if (block_count > 31) {
		printk(KERN_EMERG "%6d:%s() - overflowed source buffer\n",
		       __LINE__, __FUNCTION__);
		inbuf = (unsigned char *) ZKERNEL_START;
		inptr = 0;
		insize = 0;
		return (0);
	}

	if (!block_count) {
		nvram = (unsigned char *) ZKERNEL_START;

#ifdef INBUF_COPY
		if (!ram) {
			printk(KERN_EMERG "%6d:%s() - "
			       "ram malloc failed\n",
			       __LINE__, __FUNCTION__);
			inbuf = (unsigned char *) ZKERNEL_START;
			inptr = 0;
			insize = 0;
			return (0);
		}

		printk(KERN_DEBUG "%6d:%s() - ram buffer:0x%08x\n",
		       __LINE__, __FUNCTION__, ram);
#endif
		printk(KERN_DEBUG "%6d:%s() - ZKERNEL_START:0x%08x  "
		       "ZKERNEL_MASK:0x%08x\n",
		       __LINE__, __FUNCTION__,
		       ZKERNEL_START, ZKERNEL_MASK);
	} else {
		nvram += K64;

		while (!(ZKERNEL_MASK & (1 << block_count))) {
			printk(KERN_DEBUG "%6d:%s() - skipping block %d\n",
			       __LINE__, __FUNCTION__, block_count);

			block_count++;
			nvram += K64;

			if (block_count > 31) {
				printk(KERN_EMERG "%6d:%s() - "
				       "overflowed source buffer\n",
				       __LINE__, __FUNCTION__);
				inbuf = (unsigned char *) ZKERNEL_START;
				inptr = 0;
				insize = 1;
				return (0);
			}
		}
	}

#ifdef INBUF_COPY
	memcpy(ram, nvram, K64);
#endif
	printk(KERN_DEBUG "%6d:%s() - nvram:0x%p  block_count:%d\n",
	       __LINE__, __FUNCTION__, nvram, block_count);

#ifdef INBUF_COPY
	inbuf = ram;
#else
	inbuf = nvram;
#endif
	insize = K64;
	inptr = 1;
	intel_post(0xd0 + block_count);
	block_count++;
	return inbuf[0];
}

#else /* USE_DOC_MIL */

int fill_inbuf()
{
	extern unsigned char *inbuf;
	extern unsigned int insize;
	extern unsigned int inptr;

	if (firstfill) {
		if ((ram = malloc(K64)) == NULL) {
			printk(KERN_EMERG "%6d:%s() - ram malloc failed\n",
			      __LINE__, __FUNCTION__);
			return (0);
		}

		printk(KERN_DEBUG "%6d:%s() - ram buffer:0x%p\n",
		       __LINE__, __FUNCTION__, ram);

		block_count = 0;
		firstfill = 0;
		nvram = (unsigned char *) DOC_KERNEL_START;
	}

	memcpy_from_doc_mil(ram, nvram, K64);
	printk(KERN_DEBUG "%6d:%s() - nvram:0x%p  block_count:%d\n",
	       __LINE__, __FUNCTION__, nvram, block_count);

	nvram += K64;
	inbuf = ram;
	insize = K64;
	inptr = 1;

	intel_post(0xd0 + block_count);
	block_count++;
	return inbuf[0];
}

void memcpy_from_doc_mil(void *dest, const void *src, size_t n)
{
	int i;
	unsigned long address = (unsigned long) src;

	for (i = n; i >= 0; i -= 0x200) {
		unsigned short c = 0xffff;
		volatile unsigned char dummy;

		*(unsigned char *) (doc_mil + 0x1004) = 0x03;
		*(unsigned char *) (doc_mil + 0x800)  = 0x00;
		*(unsigned char *) (doc_mil + 0x101e) = 0x00;
		*(unsigned char *) (doc_mil + 0x1004) = 0x01;

		*(unsigned char *) (doc_mil + 0x1004) = 0x05;
		*(unsigned char *) (doc_mil + 0x800)  = address & 0xff;
		*(unsigned char *) (doc_mil + 0x800)  = (address >> 9) & 0xff;
		*(unsigned char *) (doc_mil + 0x800)  = (address >> 17) & 0xff;
		*(unsigned char *) (doc_mil + 0x101e) = 0x00;
		*(unsigned char *) (doc_mil + 0x1004) = 0x01;

		while (!(*(unsigned char *) (doc_mil + 0x1004) & 0x80) && --c) 
			/* wait for chip response */;

		dummy = *(unsigned char *) (doc_mil + 0x101d);
		memcpy(dest, doc_mil + 0x800, 0x200);

		dest += 0x200;
		address += 0x200;
	}
}
#endif /* USE_DOC_MIL */
