/*
This software and ancillary information (herein called SOFTWARE )
called LinuxBIOS          is made available under the terms described
here.  The SOFTWARE has been approved for release with associated
LA-CC Number 00-34   .  Unless otherwise indicated, this SOFTWARE has
been authored by an employee or employees of the University of
California, operator of the Los Alamos National Laboratory under
Contract No. W-7405-ENG-36 with the U.S. Department of Energy.  The
U.S. Government has rights to use, reproduce, and distribute this
SOFTWARE.  The public may copy, distribute, prepare derivative works
and publicly display this SOFTWARE without charge, provided that this
Notice and any statement of authorship are reproduced on all copies.
Neither the Government nor the University makes any warranty, express
or implied, or assumes any liability or responsibility for the use of
this SOFTWARE.  If SOFTWARE is modified to produce derivative works,
such modified SOFTWARE should be clearly marked, so as not to confuse
it with the version available from LANL.
 */
/* Copyright 2000, Ron Minnich, Advanced Computing Lab, LANL
 * rminnich@lanl.gov
 */


/*
 * C Bootstrap code for the INTEL 
 * $Id: hardwaremain.c,v 1.3 2000/10/17 22:58:51 rminnich Exp $
 *
 */

#define LINUXBIOS
#ifndef lint
static char rcsid[] = "$Id: hardwaremain.c,v 1.3 2000/10/17 22:58:51 rminnich Exp $";
#endif

#include <cpu/p5/io.h>
#include <intel.h>
#include <pciconf.h>
#include <cpu/p5/cpuid.h>
#include <subr.h>
#include <printk.h>

void intel_main()
{
  // These are only used here, so don't bother putting them in a .h
  void mainboard_fixup(void);
  void nvram_on(void);
  void keyboard_on(void);
  void framebuffer_on(void);
  void intel_copy_irq_routing_table(void);

	/* the order here is a bit tricky. We don't want to do much of 
	 * anything that uses config registers until after PciAllocateResources
	 * since that function also figures out what kind of config strategy
	 * to use (type 1 or type 2). 
	 * so we turn on cache, then worry about PCI setup, then do other 
	 * things, so that the other work can use the PciRead* and PciWrite*
	 * functions. 
	 */
	unsigned long totalram = 0;
	extern void linuxbiosmain(unsigned long membase, unsigned long totalram);
	extern void malloc_init(unsigned long start, unsigned long end);
#ifdef UPDATE_MICROCODE
	void intel_display_cpuid_microcode(void);
#endif

	intel_post(0x40);
	displayinit();

	/* segment 3,4,5,6 are used for malloc memory */
	malloc_init(0x30000, 0x70000);

	printk("Reached intel_main().\n");

#ifdef UPDATE_MICROCODE
	intel_post(0x41);

	printk("Updating microcode\n");
	intel_display_cpuid_microcode();
#endif
	intel_post(0x42);

	// This is a cheap way to buy some performance: 
	// Turn cache on the first 2M so that the next few steps
	// (before sizeram) 
	// run well. 

	// Note: can't do this on the 630! interferes with frame buffer
	// I just want light-speed booting, we have to fix the SMA problem anyway
	intel_cache_on(0, 2048);

	intel_post(0x43);

#ifdef NEWPCI
	// pick how to scan the bus. This is first so we can get at memory size.
	pci_set_method();
	intel_post(0x5f);
	pci_enumerate();
	intel_post(0x66);
#endif

	// The framebuffer can change how much memory you have. 
	// So you really need to run this before you size ram. 
#ifdef HAVE_FRAMEBUFFER
	framebuffer_on();
#endif

#ifdef NEWPCI
	printk("Using NEWPCI.\n");

	totalram = sizeram();
	intel_post(0x70);
	printk("totalram: %ldM\n", totalram/1024);

	// can't size just yet ... 
	// mainboard totalram sizing may not be up yet. If it is not ready, 
	// take a default of 64M
	if (!totalram)
		totalram = 64 * 1024;

	// Turn on cache before configuring the bus. 
	printk("Enabling cache...");
	intel_cache_on(0, totalram);
	intel_post(0x80);
	printk("done.\n");
	printk("Allocating PCI resources...\n");

	// Now do the real bus
	// we round the total ram up a lot for thing like the SISFB, which 
	// shares high memory with the CPU. 
	//configure_pci(round(totalram, 64 * 1024 * 1024), 0x1000);
	pci_configure();
	intel_post(0x88);
	pci_enable();
	intel_post(0x90);
	printk("done.\n");
#else
	printk("Using old PCI code.\n");
	printk("Allocating PCI resources...\n");

	// THIS CALL MUST BE FIRST!
	PciAllocateResources();

	printk("done.\n");
	printk("Enabling cache...");

	totalram = sizeram();
	printk("totalram: %dM\n", totalram/1024);
	// mainboard totalram sizing may not be up yet. If it is not ready, 
	// take a default of 64M

	if (!totalram)
		totalram = 64 * 1024;
	intel_cache_on(0, totalram);
	printk("done.\n");
#endif

	// generic mainboard fixup
	mainboard_fixup();

	printk("Enabling extended BIOS access...");
	nvram_on();
	printk("done.\n");

#ifndef NEWPCI
	intel_pci_list();
#endif

	intel_display_cpuid();
	intel_mtrr_check();

#ifndef NO_KEYBOARD
	keyboard_on();
#endif

#ifdef MUST_ENABLE_FLOPPY
	enable_floppy();
#endif

#ifdef SMP
	/* copy the smp block to address 0 */
	intel_smpblock(0);
#endif

#ifdef NEWPCI
	intel_zero_irq_settings();
	intel_check_irq_routing_table();
	intel_copy_irq_routing_table();
#endif

	/* to do: intel_serial_on(); */
	printk("Enabling interrupts...");
	intel_interrupts_on();
	printk("done.\n");

#ifdef LINUXBIOS
	printk("Jumping to linuxbiosmain()...");
	// we could go to argc, argv, for main but it seems like overkill.
	intel_post(0xed);
	linuxbiosmain(0, totalram);
#endif
}
